@@ -16,6 +16,8 @@ You should have received a copy of the GNU General Public License
 along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#define pr_fmt(fmt)	KBUILD_MODNAME ": " fmt
+
 #include <linux/compat.h>
 #include <linux/cdev.h>
 #include <asm/syscall.h>
@@ -43,21 +45,22 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #define compat_ptr(X) X
 #endif
 
+/*
+ * The kernel patched with grsecurity makes the default access_ok trigger a
+ * might_sleep(), so if present we use the one defined by them
+ */
+#ifdef access_ok_noprefault
+#define ppm_access_ok access_ok_noprefault
+#else
+#define ppm_access_ok access_ok
+#endif
+
 static void memory_dump(char *p, size_t size)
 {
 	unsigned int j;
 
-	for (j = 0; j < size; j += 8) {
-		pr_info("%.2x %.2x %.2x %.2x %.2x %.2x %.2x %.2x\n",
-		       (uint8_t)p[j],
-		       (uint8_t)p[j + 1],
-		       (uint8_t)p[j + 2],
-		       (uint8_t)p[j + 3],
-		       (uint8_t)p[j + 4],
-		       (uint8_t)p[j + 5],
-		       (uint8_t)p[j + 6],
-		       (uint8_t)p[j + 7]);
-	}
+	for (j = 0; j < size; j += 8)
+		pr_info("%*ph\n", 8, &p[j]);
 }
 
 /*
@@ -74,9 +77,8 @@ unsigned long ppm_copy_from_user(void *to, const void __user *from, unsigned lon
 
 	pagefault_disable();
 
-	if (likely(access_ok(VERIFY_READ, from, n))) {
+	if (likely(ppm_access_ok(VERIFY_READ, from, n)))
 		res = __copy_from_user_inatomic(to, from, n);
-	}
 
 	pagefault_enable();
 
@@ -103,11 +105,10 @@ long ppm_strncpy_from_user(char *to, const char __user *from, unsigned long n)
 		 * Read bytes_to_read bytes at a time, and look for the terminator. Should be fast
 		 * since the copy_from_user is optimized for the processor
 		 */
-		if (n < bytes_to_read) {
+		if (n < bytes_to_read)
 			bytes_to_read = n;
-		}
 
-		if (!access_ok(VERIFY_READ, from, n)) {
+		if (!ppm_access_ok(VERIFY_READ, from, n)) {
 			res = -1;
 			goto strncpy_end;
 		}
@@ -144,17 +145,19 @@ strncpy_end:
  * NOTES:
  * - val_len is ignored for everything other than PT_BYTEBUF.
  * - fromuser is ignored for numeric types
+ * - dyn_idx is ignored for everything other than PT_DYN
  */
-inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, uint16_t val_len, bool fromuser)
+inline int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len, bool fromuser, u8 dyn_idx)
 {
-	int32_t len = -1;
-	uint16_t *psize = (uint16_t *)(args->buffer + args->curarg * sizeof(uint16_t));
+	const struct ppm_param_info* param_info;
+	int len = -1;
+	u16 *psize = (u16 *)(args->buffer + args->curarg * sizeof(u16));
 
 	if (unlikely(args->curarg >= args->nargs)) {
-		pr_info("sysdig-probe: %u)val_to_ring: too many arguments for event #%u, type=%u, curarg=%u, nargs=%u tid:%u\n",
+		pr_err("(%u)val_to_ring: too many arguments for event #%u, type=%u, curarg=%u, nargs=%u tid:%u\n",
 		       smp_processor_id(),
 		       args->nevents,
-		       (uint32_t)args->event_type,
+		       (u32)args->event_type,
 		       args->curarg,
 		       args->nargs,
 		       current->pid);
@@ -163,7 +166,36 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 		return PPM_FAILURE_BUG;
 	}
 
-	switch (g_event_info[args->event_type].params[args->curarg].type) {
+	if (unlikely(args->arg_data_size == 0)) {
+		return PPM_FAILURE_BUFFER_FULL;
+	}
+
+	param_info = &(g_event_info[args->event_type].params[args->curarg]);
+	if (param_info->type == PT_DYN && param_info->info != NULL) {
+		const struct ppm_param_info *dyn_params;
+
+		if (unlikely(dyn_idx >= param_info->ninfo)) {
+			ASSERT(0);
+			return PPM_FAILURE_BUG;
+		}
+
+		dyn_params = (const struct ppm_param_info *)param_info->info;
+
+		param_info = &dyn_params[dyn_idx];
+		if (likely(args->arg_data_size >= sizeof(u8)))	{
+			*(u8 *)(args->buffer + args->arg_data_offset) = dyn_idx;
+			len = sizeof(u8);
+		} else {
+			return PPM_FAILURE_BUFFER_FULL;
+		}
+		args->arg_data_offset += len;
+		args->arg_data_size -= len;
+		*psize = (u16)len;
+	} else {
+		*psize = 0;
+	}
+
+	switch (param_info->type) {
 	case PT_CHARBUF:
 	case PT_FSPATH:
 		if (likely(val != 0)) {
@@ -171,16 +203,13 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 				len = ppm_strncpy_from_user(args->buffer + args->arg_data_offset,
 					(const char __user *)(unsigned long)val, args->arg_data_size);
 
-				if (unlikely(len < 0)) {
+				if (unlikely(len < 0))
 					return PPM_FAILURE_INVALID_USER_MEMORY;
-				}
 			} else {
-				char *dest = strncpy(args->buffer + args->arg_data_offset,
+				len = strlcpy(args->buffer + args->arg_data_offset,
 								(const char *)(unsigned long)val,
 								args->arg_data_size);
-
-				dest[args->arg_data_size - 1] = 0;
-				len = strlen(dest) + 1;
+				++len;
 			}
 
 			/*
@@ -191,12 +220,10 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 			/*
 			 * Handle NULL pointers
 			 */
-			char *dest = strncpy(args->buffer + args->arg_data_offset,
+			len = strlcpy(args->buffer + args->arg_data_offset,
 			       "(NULL)",
 			       args->arg_data_size);
-
-			dest[args->arg_data_size - 1] = 0;
-			len = strlen(dest) + 1;
+			++len;
 		}
 
 		break;
@@ -209,13 +236,12 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 				return PPM_FAILURE_BUFFER_FULL;
 			} else {
 				if (fromuser) {
-					len = (int32_t)ppm_copy_from_user(args->buffer + args->arg_data_offset,
+					len = (int)ppm_copy_from_user(args->buffer + args->arg_data_offset,
 							(const void __user *)(unsigned long)val,
 							val_len);
 
-					if (unlikely(len != 0)) {
+					if (unlikely(len != 0))
 						return PPM_FAILURE_INVALID_USER_MEMORY;
-					}
 
 					len = val_len;
 				} else {
@@ -236,9 +262,9 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 	case PT_FLAGS8:
 	case PT_UINT8:
 	case PT_SIGTYPE:
-		if (likely(args->arg_data_size >= sizeof(uint8_t)))	{
-			*(uint8_t *)(args->buffer + args->arg_data_offset) = (uint8_t)val;
-			len = sizeof(uint8_t);
+		if (likely(args->arg_data_size >= sizeof(u8)))	{
+			*(u8 *)(args->buffer + args->arg_data_offset) = (u8)val;
+			len = sizeof(u8);
 		} else {
 			return PPM_FAILURE_BUFFER_FULL;
 		}
@@ -247,9 +273,9 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 	case PT_FLAGS16:
 	case PT_UINT16:
 	case PT_SYSCALLID:
-		if (likely(args->arg_data_size >= sizeof(uint16_t))) {
-			*(uint16_t *)(args->buffer + args->arg_data_offset) = (uint16_t)val;
-			len = sizeof(uint16_t);
+		if (likely(args->arg_data_size >= sizeof(u16))) {
+			*(u16 *)(args->buffer + args->arg_data_offset) = (u16)val;
+			len = sizeof(u16);
 		} else {
 			return PPM_FAILURE_BUFFER_FULL;
 		}
@@ -257,9 +283,9 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 		break;
 	case PT_FLAGS32:
 	case PT_UINT32:
-		if (likely(args->arg_data_size >= sizeof(uint32_t))) {
-			*(uint32_t *)(args->buffer + args->arg_data_offset) = (uint32_t)val;
-			len = sizeof(uint32_t);
+		if (likely(args->arg_data_size >= sizeof(u32))) {
+			*(u32 *)(args->buffer + args->arg_data_offset) = (u32)val;
+			len = sizeof(u32);
 		} else {
 			return PPM_FAILURE_BUFFER_FULL;
 		}
@@ -268,36 +294,36 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 	case PT_RELTIME:
 	case PT_ABSTIME:
 	case PT_UINT64:
-		if (likely(args->arg_data_size >= sizeof(uint64_t))) {
-			*(uint64_t *)(args->buffer + args->arg_data_offset) = (uint64_t)val;
-			len = sizeof(uint64_t);
+		if (likely(args->arg_data_size >= sizeof(u64))) {
+			*(u64 *)(args->buffer + args->arg_data_offset) = (u64)val;
+			len = sizeof(u64);
 		} else {
 			return PPM_FAILURE_BUFFER_FULL;
 		}
 
 		break;
 	case PT_INT8:
-		if (likely(args->arg_data_size >= sizeof(int8_t))) {
-			*(int8_t *)(args->buffer + args->arg_data_offset) = (int8_t)(long)val;
-			len = sizeof(int8_t);
+		if (likely(args->arg_data_size >= sizeof(s8))) {
+			*(s8 *)(args->buffer + args->arg_data_offset) = (s8)(long)val;
+			len = sizeof(s8);
 		} else {
 			return PPM_FAILURE_BUFFER_FULL;
 		}
 
 		break;
 	case PT_INT16:
-		if (likely(args->arg_data_size >= sizeof(int16_t))) {
-			*(int16_t *)(args->buffer + args->arg_data_offset) = (int16_t)(long)val;
-			len = sizeof(int16_t);
+		if (likely(args->arg_data_size >= sizeof(s16))) {
+			*(s16 *)(args->buffer + args->arg_data_offset) = (s16)(long)val;
+			len = sizeof(s16);
 		} else {
 			return PPM_FAILURE_BUFFER_FULL;
 		}
 
 		break;
 	case PT_INT32:
-		if (likely(args->arg_data_size >= sizeof(int32_t))) {
-			*(int32_t *)(args->buffer + args->arg_data_offset) = (int32_t)(long)val;
-			len = sizeof(int32_t);
+		if (likely(args->arg_data_size >= sizeof(s32))) {
+			*(s32 *)(args->buffer + args->arg_data_offset) = (s32)(long)val;
+			len = sizeof(s32);
 		} else {
 			return PPM_FAILURE_BUFFER_FULL;
 		}
@@ -307,9 +333,9 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 	case PT_ERRNO:
 	case PT_FD:
 	case PT_PID:
-		if (likely(args->arg_data_size >= sizeof(int64_t))) {
-			*(int64_t *)(args->buffer + args->arg_data_offset) = (int64_t)(long)val;
-			len = sizeof(int64_t);
+		if (likely(args->arg_data_size >= sizeof(s64))) {
+			*(s64 *)(args->buffer + args->arg_data_offset) = (s64)(long)val;
+			len = sizeof(s64);
 		} else {
 			return PPM_FAILURE_BUFFER_FULL;
 		}
@@ -317,9 +343,9 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 		break;
 	default:
 		ASSERT(0);
-		pr_info("sysdig-probe: val_to_ring: invalid argument type %d. Event %u (%s) might have less parameters than what has been declared in nparams\n",
+		pr_err("val_to_ring: invalid argument type %d. Event %u (%s) might have less parameters than what has been declared in nparams\n",
 		       (int)g_event_info[args->event_type].params[args->curarg].type,
-		       (uint32_t)args->event_type,
+		       (u32)args->event_type,
 		       g_event_info[args->event_type].name);
 		return PPM_FAILURE_BUG;
 	}
@@ -327,7 +353,7 @@ inline int32_t val_to_ring(struct event_filler_arguments *args, uint64_t val, ui
 	ASSERT(len <= 65535);
 	ASSERT(len <= args->arg_data_size);
 
-	*psize = (uint16_t)len;
+	*psize += (u16)len;
 	args->curarg++;
 	args->arg_data_offset += len;
 	args->arg_data_size -= len;
@@ -348,7 +374,7 @@ char *npm_getcwd(char *buf, unsigned long bufsize)
 
 	ASSERT(bufsize >= PAGE_SIZE - 1);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) || defined CONFIG_VE
 	get_fs_pwd(current->fs, &pwd);
 #else
 	read_lock(&current->fs->lock);
@@ -358,12 +384,16 @@ char *npm_getcwd(char *buf, unsigned long bufsize)
 #endif
 
 	res = d_path(&pwd, buf, bufsize);
+
+	if (IS_ERR(res))
+		res = NULL;
+
 	path_put(&pwd);
 
 	return res;
 }
 
-static inline uint8_t socket_family_to_scap(uint8_t family)
+static inline u8 socket_family_to_scap(u8 family)
 {
 	if (family == AF_INET) {
 		return PPM_AF_INET;
@@ -481,18 +511,18 @@ static struct socket *ppm_sockfd_lookup_light(int fd, int *err, int *fput_needed
  * Convert a sockaddr into our address representation and copy it to
  * targetbuf
  */
-uint16_t pack_addr(struct sockaddr *usrsockaddr,
+u16 pack_addr(struct sockaddr *usrsockaddr,
 	int ulen,
 	char *targetbuf,
-	uint16_t targetbufsize)
+	u16 targetbufsize)
 {
-	uint32_t ip;
-	uint16_t port;
+	u32 ip;
+	u16 port;
 	sa_family_t family = usrsockaddr->sa_family;
 	struct sockaddr_in *usrsockaddr_in;
 	struct sockaddr_in6 *usrsockaddr_in6;
 	struct sockaddr_un *usrsockaddr_un;
-	uint16_t size;
+	u16 size;
 	char *dest;
 
 	switch (family) {
@@ -514,8 +544,8 @@ uint16_t pack_addr(struct sockaddr *usrsockaddr,
 		size = 1 + 4 + 2; /* family + ip + port */
 
 		*targetbuf = socket_family_to_scap(family);
-		*(uint32_t *)(targetbuf + 1) = ip;
-		*(uint16_t *)(targetbuf + 5) = port;
+		*(u32 *)(targetbuf + 1) = ip;
+		*(u16 *)(targetbuf + 5) = port;
 
 		break;
 	case AF_INET6:
@@ -538,7 +568,7 @@ uint16_t pack_addr(struct sockaddr *usrsockaddr,
 		memcpy(targetbuf + 1,
 			usrsockaddr_in6->sin6_addr.s6_addr,
 			16);
-		*(uint16_t *)(targetbuf + 17) = port;
+		*(u16 *)(targetbuf + 17) = port;
 
 		break;
 	case AF_UNIX:
@@ -551,11 +581,10 @@ uint16_t pack_addr(struct sockaddr *usrsockaddr,
 		 * Put a 0 at the end of struct sockaddr_un because
 		 * the user might not have considered it in the length
 		 */
-		if (ulen == sizeof(struct sockaddr_storage)) {
+		if (ulen == sizeof(struct sockaddr_storage))
 			*(((char *)usrsockaddr_un) + ulen - 1) = 0;
-		} else {
+		else
 			*(((char *)usrsockaddr_un) + ulen) = 0;
-		}
 
 		/*
 		 * Pack the data into the target buffer
@@ -583,13 +612,13 @@ uint16_t pack_addr(struct sockaddr *usrsockaddr,
  * Convert a connection tuple into our tuple representation and copy it to
  * targetbuf
  */
-uint16_t fd_to_socktuple(int fd,
+u16 fd_to_socktuple(int fd,
 	struct sockaddr *usrsockaddr,
 	int ulen,
 	bool use_userdata,
 	bool is_inbound,
 	char *targetbuf,
-	uint16_t targetbufsize)
+	u16 targetbufsize)
 {
 	struct socket *sock;
 	int err = 0;
@@ -597,16 +626,16 @@ uint16_t fd_to_socktuple(int fd,
 	struct unix_sock *us;
 	char *us_name;
 	struct sock *speer;
-	uint32_t sip;
-	uint32_t dip;
-	uint8_t *sip6;
-	uint8_t *dip6;
-	uint16_t sport;
-	uint16_t dport;
+	u32 sip;
+	u32 dip;
+	u8 *sip6;
+	u8 *dip6;
+	u16 sport;
+	u16 dport;
 	struct sockaddr_in *usrsockaddr_in;
 	struct sockaddr_in6 *usrsockaddr_in6;
 	struct sockaddr_un *usrsockaddr_un;
-	uint16_t size;
+	u16 size;
 	char *dest;
 	struct sockaddr_storage sock_address;
 	struct sockaddr_storage peer_address;
@@ -624,9 +653,8 @@ uint16_t fd_to_socktuple(int fd,
 		 * This usually happens if the call failed without being able to establish a connection,
 		 * i.e. if it didn't return something like SE_EINPROGRESS.
 		 */
-		if (sock) {
+		if (sock)
 			sockfd_put(sock);
-		}
 		return 0;
 	}
 
@@ -680,10 +708,10 @@ uint16_t fd_to_socktuple(int fd,
 		size = 1 + 4 + 4 + 2 + 2; /* family + sip + dip + sport + dport */
 
 		*targetbuf = socket_family_to_scap(family);
-		*(uint32_t *)(targetbuf + 1) = sip;
-		*(uint16_t *)(targetbuf + 5) = sport;
-		*(uint32_t *)(targetbuf + 7) = dip;
-		*(uint16_t *)(targetbuf + 11) = dport;
+		*(u32 *)(targetbuf + 1) = sip;
+		*(u16 *)(targetbuf + 5) = sport;
+		*(u32 *)(targetbuf + 7) = dip;
+		*(u16 *)(targetbuf + 11) = dport;
 
 		break;
 	case AF_INET6:
@@ -730,11 +758,11 @@ uint16_t fd_to_socktuple(int fd,
 		memcpy(targetbuf + 1,
 			sip6,
 			16);
-		*(uint16_t *)(targetbuf + 17) = sport;
+		*(u16 *)(targetbuf + 17) = sport;
 		memcpy(targetbuf + 19,
 			dip6,
 			16);
-		*(uint16_t *)(targetbuf + 35) = dport;
+		*(u16 *)(targetbuf + 35) = dport;
 
 		break;
 	case AF_UNIX:
@@ -778,17 +806,15 @@ uint16_t fd_to_socktuple(int fd,
 			 * Put a 0 at the end of struct sockaddr_un because
 			 * the user might not have considered it in the length
 			 */
-			if (ulen == sizeof(struct sockaddr_storage)) {
+			if (ulen == sizeof(struct sockaddr_storage))
 				*(((char *)usrsockaddr_un) + ulen - 1) = 0;
-			} else {
+			else
 				*(((char *)usrsockaddr_un) + ulen) = 0;
-			}
 
-			if (is_inbound) {
+			if (is_inbound)
 				us_name = ((struct sockaddr_un *) &sock_address)->sun_path;
-			} else {
+			else
 				us_name = usrsockaddr_un->sun_path;
-			}
 		}
 
 		ASSERT(us_name);
@@ -814,17 +840,14 @@ uint16_t fd_to_socktuple(int fd,
 
 int addr_to_kernel(void __user *uaddr, int ulen, struct sockaddr *kaddr)
 {
-	if (unlikely(ulen < 0 || ulen > sizeof(struct sockaddr_storage))) {
+	if (unlikely(ulen < 0 || ulen > sizeof(struct sockaddr_storage)))
 		return -EINVAL;
-	}
 
-	if (unlikely(ulen == 0)) {
+	if (unlikely(ulen == 0))
 		return 0;
-	}
 
-	if (unlikely(ppm_copy_from_user(kaddr, uaddr, ulen))) {
+	if (unlikely(ppm_copy_from_user(kaddr, uaddr, ulen)))
 		return -EFAULT;
-	}
 
 	return 0;
 }
@@ -837,21 +860,19 @@ int32_t parse_readv_writev_bufs(struct event_filler_arguments *args, const struc
 {
 	int32_t res;
 	const struct iovec *iov;
-	uint32_t copylen;
-	uint32_t j;
+	u32 copylen;
+	u32 j;
 	uint64_t size = 0;
 	unsigned long bufsize;
 	char *targetbuf = args->str_storage;
 
 	copylen = iovcnt * sizeof(struct iovec);
 
-	if (unlikely(copylen >= STR_STORAGE_SIZE)) {
+	if (unlikely(copylen >= STR_STORAGE_SIZE))
 		return PPM_FAILURE_BUFFER_FULL;
-	}
 
-	if (unlikely(ppm_copy_from_user(targetbuf, iovsrc, copylen))) {
+	if (unlikely(ppm_copy_from_user(targetbuf, iovsrc, copylen)))
 		return PPM_FAILURE_INVALID_USER_MEMORY;
-	}
 
 	iov = (const struct iovec *)targetbuf;
 
@@ -859,14 +880,12 @@ int32_t parse_readv_writev_bufs(struct event_filler_arguments *args, const struc
 	 * Size
 	 */
 	if (flags & PRB_FLAG_PUSH_SIZE) {
-		for (j = 0; j < iovcnt; j++) {
+		for (j = 0; j < iovcnt; j++)
 			size += iov[j].iov_len;
-		}
 
-		res = val_to_ring(args, size, 0, false);
-		if (unlikely(res != PPM_SUCCESS)) {
+		res = val_to_ring(args, size, 0, false, 0);
+		if (unlikely(res != PPM_SUCCESS))
 			return res;
-		}
 	}
 
 	/*
@@ -882,15 +901,14 @@ int32_t parse_readv_writev_bufs(struct event_filler_arguments *args, const struc
 			res = val_to_ring(args,
 				(unsigned long)iov[0].iov_base,
 				min(bufsize, (unsigned long)g_snaplen),
-				true);
-			if (unlikely(res != PPM_SUCCESS)) {
+				true,
+				0);
+			if (unlikely(res != PPM_SUCCESS))
 				return res;
-			}
 		} else {
-			res = val_to_ring(args, 0, 0, false);
-			if (unlikely(res != PPM_SUCCESS)) {
+			res = val_to_ring(args, 0, 0, false, 0);
+			if (unlikely(res != PPM_SUCCESS))
 				return res;
-			}
 		}
 	}
 
@@ -908,18 +926,18 @@ int32_t parse_readv_writev_bufs(struct event_filler_arguments *args, const struc
  * filler function.
  * The arguments to extract are be specified in g_ppm_events.
  */
-int32_t f_sys_autofill(struct event_filler_arguments *args, const struct ppm_event_entry *evinfo)
+int f_sys_autofill(struct event_filler_arguments *args, const struct ppm_event_entry *evinfo)
 {
-	int32_t res;
+	int res;
 	unsigned long val;
-	uint32_t j;
+	u32 j;
 	int64_t retval;
 
 	ASSERT(evinfo->n_autofill_args <= PPM_MAX_AUTOFILL_ARGS);
 
 	for (j = 0; j < evinfo->n_autofill_args; j++) {
 		if (evinfo->autofill_args[j].id >= 0) {
-#ifdef __x86_64__
+#ifndef __NR_socketcall
 			/*
 			 * Regular argument
 			 */
@@ -943,27 +961,24 @@ int32_t f_sys_autofill(struct event_filler_arguments *args, const struct ppm_eve
 			}
 #endif
 
-			res = val_to_ring(args, val, 0, true);
-			if (unlikely(res != PPM_SUCCESS)) {
+			res = val_to_ring(args, val, 0, true, 0);
+			if (unlikely(res != PPM_SUCCESS))
 				return res;
-			}
 		} else if (evinfo->autofill_args[j].id == AF_ID_RETVAL) {
 			/*
 			 * Return value
 			 */
 			retval = (int64_t)(long)syscall_get_return_value(current, args->regs);
-			res = val_to_ring(args, retval, 0, false);
-			if (unlikely(res != PPM_SUCCESS)) {
+			res = val_to_ring(args, retval, 0, false, 0);
+			if (unlikely(res != PPM_SUCCESS))
 				return res;
-			}
 		} else if (evinfo->autofill_args[j].id == AF_ID_USEDEFAULT) {
 			/*
 			 * Default Value
 			 */
-			res = val_to_ring(args, evinfo->autofill_args[j].default_val, 0, false);
-			if (unlikely(res != PPM_SUCCESS)) {
+			res = val_to_ring(args, evinfo->autofill_args[j].default_val, 0, false, 0);
+			if (unlikely(res != PPM_SUCCESS))
 				return res;
-			}
 		} else {
 			ASSERT(false);
 		}
