@@ -19,6 +19,10 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef EVENTS_PUBLIC_H_
 #define EVENTS_PUBLIC_H_
 
+#if defined(__sun)
+#include <sys/ioccom.h>
+#endif
+
 #ifdef __KERNEL__
 #include <linux/types.h>
 #endif
@@ -208,9 +212,11 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #define PPM_FCNTL_F_GETOWN 12
 #define PPM_FCNTL_F_SETSIG 13
 #define PPM_FCNTL_F_GETSIG 15
+#ifndef CONFIG_64BIT
 #define PPM_FCNTL_F_GETLK64 17
 #define PPM_FCNTL_F_SETLK64 18
 #define PPM_FCNTL_F_SETLKW64 19
+#endif
 #define PPM_FCNTL_F_SETOWN_EX 21
 #define PPM_FCNTL_F_GETOWN_EX 22
 #define PPM_FCNTL_F_SETLEASE 23
@@ -221,6 +227,96 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #define PPM_FCNTL_F_SETPIPE_SZ 28
 #define PPM_FCNTL_F_GETPIPE_SZ 29
 
+ /*
+ * ptrace requests
+ */
+#define PPM_PTRACE_UNKNOWN 0
+#define PPM_PTRACE_TRACEME 1
+#define PPM_PTRACE_PEEKTEXT 2
+#define PPM_PTRACE_PEEKDATA 3
+#define PPM_PTRACE_PEEKUSR 4
+#define PPM_PTRACE_POKETEXT 5
+#define PPM_PTRACE_POKEDATA 6
+#define PPM_PTRACE_POKEUSR 7
+#define PPM_PTRACE_CONT 8
+#define PPM_PTRACE_KILL 9
+#define PPM_PTRACE_SINGLESTEP 10
+#define PPM_PTRACE_ATTACH 11
+#define PPM_PTRACE_DETACH 12
+#define PPM_PTRACE_SYSCALL 13
+#define PPM_PTRACE_SETOPTIONS 14
+#define PPM_PTRACE_GETEVENTMSG 15
+#define PPM_PTRACE_GETSIGINFO 16
+#define PPM_PTRACE_SETSIGINFO 17
+#define PPM_PTRACE_GETREGSET 18
+#define PPM_PTRACE_SETREGSET 19
+#define PPM_PTRACE_SEIZE 20
+#define PPM_PTRACE_INTERRUPT 21
+#define PPM_PTRACE_LISTEN 22
+#define PPM_PTRACE_PEEKSIGINFO 23
+#define PPM_PTRACE_GETSIGMASK 24
+#define PPM_PTRACE_SETSIGMASK 25
+#define PPM_PTRACE_GETREGS 26
+#define PPM_PTRACE_SETREGS 27
+#define PPM_PTRACE_GETFPREGS 28
+#define PPM_PTRACE_SETFPREGS 29
+#define PPM_PTRACE_GETFPXREGS 30
+#define PPM_PTRACE_SETFPXREGS 31
+#define PPM_PTRACE_OLDSETOPTIONS 32
+#define PPM_PTRACE_GET_THREAD_AREA 33
+#define PPM_PTRACE_SET_THREAD_AREA 34
+#define PPM_PTRACE_ARCH_PRCTL 35
+#define PPM_PTRACE_SYSEMU 36
+#define PPM_PTRACE_SYSEMU_SINGLESTEP 37
+#define PPM_PTRACE_SINGLEBLOCK 38
+
+/*
+ * ptrace dynamic table indexes
+ */
+#define PPM_PTRACE_IDX_UINT64 0
+#define PPM_PTRACE_IDX_SIGTYPE 1
+
+#define PPM_PTRACE_IDX_MAX 2
+
+/*
+ * memory protection flags
+ */
+#define PPM_PROT_NONE		0
+#define PPM_PROT_READ		(1 << 0)
+#define PPM_PROT_WRITE		(1 << 1)
+#define PPM_PROT_EXEC		(1 << 2)
+#define PPM_PROT_SEM		(1 << 3)
+#define PPM_PROT_GROWSDOWN	(1 << 4)
+#define PPM_PROT_GROWSUP	(1 << 5)
+#define PPM_PROT_SAO		(1 << 6)
+
+/*
+ * mmap flags
+ */
+#define PPM_MAP_SHARED		(1 << 0)
+#define PPM_MAP_PRIVATE		(1 << 1)
+#define PPM_MAP_FIXED		(1 << 2)
+#define PPM_MAP_ANONYMOUS	(1 << 3)
+#define PPM_MAP_32BIT		(1 << 4)
+#define PPM_MAP_RENAME		(1 << 5)
+#define PPM_MAP_NORESERVE	(1 << 6)
+#define PPM_MAP_POPULATE	(1 << 7)
+#define PPM_MAP_NONBLOCK	(1 << 8)
+#define PPM_MAP_GROWSDOWN	(1 << 9)
+#define PPM_MAP_DENYWRITE	(1 << 10)
+#define PPM_MAP_EXECUTABLE	(1 << 11)
+#define PPM_MAP_INHERIT		(1 << 12)
+#define PPM_MAP_FILE		(1 << 13)
+#define PPM_MAP_LOCKED		(1 << 14)
+
+/*
+ * splice flags
+ */
+#define PPM_SPLICE_F_MOVE		(1 << 0)
+#define PPM_SPLICE_F_NONBLOCK	(1 << 1)
+#define PPM_SPLICE_F_MORE		(1 << 2)
+#define PPM_SPLICE_F_GIFT		(1 << 3)
+
 /*
  * SuS says limits have to be unsigned.
  * Which makes a ton more sense anyway.
@@ -264,12 +360,12 @@ enum ppm_event_type {
 	PPME_SYSCALL_READ_X = 7,
 	PPME_SYSCALL_WRITE_E = 8,
 	PPME_SYSCALL_WRITE_X = 9,
-	PPME_SYSCALL_BRK_E = 10,
-	PPME_SYSCALL_BRK_X = 11,
-	PPME_SYSCALL_EXECVE_E = 12,
-	PPME_SYSCALL_EXECVE_X = 13,
-	PPME_CLONE_E = 14,
-	PPME_CLONE_X = 15,
+	PPME_SYSCALL_BRK_1_E = 10,
+	PPME_SYSCALL_BRK_1_X = 11,
+	PPME_SYSCALL_EXECVE_8_E = 12,
+	PPME_SYSCALL_EXECVE_8_X = 13,
+	PPME_CLONE_11_E = 14,
+	PPME_CLONE_11_X = 15,
 	PPME_PROCEXIT_E = 16,
 	PPME_PROCEXIT_X = 17,	/* This should never be called */
 	PPME_SOCKET_SOCKET_E = 18,
@@ -400,15 +496,31 @@ enum ppm_event_type {
 	PPME_SYSCALL_SETRLIMIT_X = 143,
 	PPME_SYSCALL_PRLIMIT_E = 144,
 	PPME_SYSCALL_PRLIMIT_X = 145,
-	PPME_SCHEDSWITCH_E = 146,
-	PPME_SCHEDSWITCH_X = 147,	/* This should never be called */
+	PPME_SCHEDSWITCH_1_E = 146,
+	PPME_SCHEDSWITCH_1_X = 147,	/* This should never be called */
 	PPME_DROP_E = 148,  /* For internal use */
 	PPME_DROP_X = 149,	/* For internal use */
 	PPME_SYSCALL_FCNTL_E = 150,  /* For internal use */
 	PPME_SYSCALL_FCNTL_X = 151,	/* For internal use */
-	PPME_SCHEDSWITCHEX_E = 152,
-	PPME_SCHEDSWITCHEX_X = 153,	/* This should never be called */
-	PPM_EVENT_MAX = 154,
+	PPME_SCHEDSWITCH_6_E = 152,
+	PPME_SCHEDSWITCH_6_X = 153,	/* This should never be called */
+	PPME_SYSCALL_EXECVE_13_E = 154,
+	PPME_SYSCALL_EXECVE_13_X = 155,
+	PPME_CLONE_16_E = 156,
+	PPME_CLONE_16_X = 157,
+	PPME_SYSCALL_BRK_4_E = 158,
+	PPME_SYSCALL_BRK_4_X = 159,
+	PPME_SYSCALL_MMAP_E = 160,
+	PPME_SYSCALL_MMAP_X = 161,
+	PPME_SYSCALL_MMAP2_E = 162,
+	PPME_SYSCALL_MMAP2_X = 163,
+	PPME_SYSCALL_MUNMAP_E = 164,
+	PPME_SYSCALL_MUNMAP_X = 165,
+	PPME_SYSCALL_SPLICE_E = 166,
+	PPME_SYSCALL_SPLICE_X = 167,
+	PPME_SYSCALL_PTRACE_E = 168,
+	PPME_SYSCALL_PTRACE_X = 169,
+	PPM_EVENT_MAX = 170
 };
 /*@}*/
 
@@ -712,7 +824,14 @@ enum ppm_syscall_code {
 	PPM_SC_WAITPID = 293,
 	PPM_SC_PREAD64 = 294,
 	PPM_SC_PWRITE64 = 295,
-	PPM_SC_MAX = 296,
+	PPM_SC_ARCH_PRCTL = 296,
+	PPM_SC_SHMAT = 297,
+	PPM_SC_SIGRETURN = 298,
+	PPM_SC_FALLOCATE = 299,
+	PPM_SC_NEWFSSTAT = 300,
+	PPM_SC_PROCESS_VM_READV = 301,
+	PPM_SC_PROCESS_VM_WRITEV = 302,
+	PPM_SC_MAX = 303,
 };
 
 /*
@@ -826,7 +945,9 @@ struct ppm_param_info {
 	char name[PPM_MAX_NAME_LEN];  /**< Paramter name, e.g. 'size'. */
 	enum ppm_param_type type; /**< Paramter type, e.g. 'uint16', 'string'... */
 	enum ppm_print_format fmt; /**< If this is a numeric parameter, this flag specifies if it should be rendered as decimal or hex. */
-	const struct ppm_name_value *symbols; /**< If this is a flags parameter, it points to an array of ppm_name_value, terminated with {0, 0} */
+	const void *info; /**< If this is a flags parameter, it points to an array of ppm_name_value,
+			       else if this is a dynamic parameter it points to an array of ppm_param_info */
+	uint8_t ninfo; /**< Number of entry in the info array. */
 };
 
 /*!
@@ -846,6 +967,8 @@ struct ppm_event_info {
 #if defined _MSC_VER
 #pragma pack(push)
 #pragma pack(1)
+#elif defined __sun
+#pragma pack(1)
 #else
 #pragma pack(push, 1)
 #endif
@@ -857,9 +980,12 @@ struct ppm_evt_hdr {
 	uint64_t tid; /* the tid of the thread that generated this event */
 	uint32_t len; /* the event len, including the header */
 	uint16_t type; /* the event type */
-/* uint16_t cpuid; the cpu that generated the event */
 };
+#if defined __sun
+#pragma pack()
+#else
 #pragma pack(pop)
+#endif
 
 /*
  * IOCTL codes
@@ -870,6 +996,9 @@ struct ppm_evt_hdr {
 #define PPM_IOCTL_DISABLE_DROPPING_MODE _IO(PPM_IOCTL_MAGIC, 2)
 #define PPM_IOCTL_ENABLE_DROPPING_MODE _IO(PPM_IOCTL_MAGIC, 3)
 #define PPM_IOCTL_SET_SNAPLEN _IO(PPM_IOCTL_MAGIC, 4)
+#define PPM_IOCTL_MASK_ZERO_EVENTS _IO(PPM_IOCTL_MAGIC, 5)
+#define PPM_IOCTL_MASK_SET_EVENT   _IO(PPM_IOCTL_MAGIC, 6)
+#define PPM_IOCTL_MASK_UNSET_EVENT _IO(PPM_IOCTL_MAGIC, 7)
 
 
 /*!
@@ -887,8 +1016,13 @@ extern const struct ppm_name_value futex_operations[];
 extern const struct ppm_name_value lseek_whence[];
 extern const struct ppm_name_value poll_flags[];
 extern const struct ppm_name_value shutdown_how[];
-extern const struct ppm_name_value openat_flags[];
 extern const struct ppm_name_value rlimit_resources[];
 extern const struct ppm_name_value fcntl_commands[];
+extern const struct ppm_name_value ptrace_requests[];
+extern const struct ppm_name_value prot_flags[];
+extern const struct ppm_name_value mmap_flags[];
+extern const struct ppm_name_value splice_flags[];
+
+extern const struct ppm_param_info ptrace_dynamic_param[];
 
 #endif /* EVENTS_PUBLIC_H_ */
