@@ -25,6 +25,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <poll.h>
+#include <errno.h>
 #endif // _WIN32
 
 #include "scap.h"
@@ -42,11 +43,8 @@ char* scap_getlasterr(scap_t* handle)
 
 scap_t* scap_open_live(char *error)
 {
-#ifdef _WIN32
-	snprintf(error, SCAP_LASTERR_SIZE, "live capture not supported on windows");
-	return NULL;
-#elif defined(__APPLE__)
-	snprintf(error, SCAP_LASTERR_SIZE, "live capture not supported on OSX");
+#if !defined(HAS_CAPTURE)
+	snprintf(error, SCAP_LASTERR_SIZE, "live capture not supported on %s", PLATFORM_NAME);
 	return NULL;
 #else
 	uint32_t j;
@@ -77,6 +75,7 @@ scap_t* scap_open_live(char *error)
 	handle->m_addrlist = NULL;
 	handle->m_userlist = NULL;
 	handle->m_emptybuf_timeout_ms = BUFFER_EMPTY_WAIT_TIME_MS;
+	handle->m_last_evt_dump_flags = 0;
 
 	//
 	// Find out how many devices we have to open, which equals to the number of CPUs
@@ -96,17 +95,6 @@ scap_t* scap_open_live(char *error)
 		return NULL;
 	}
 
-	//
-	// Allocate the array of poll fds.
-	//
-	handle->m_pollfds = (struct pollfd*)malloc(ndevs * sizeof(struct pollfd));
-	if(!handle->m_pollfds)
-	{
-		scap_close(handle);
-		snprintf(error, SCAP_LASTERR_SIZE, "error allocating the device handles");
-		return NULL;
-	}
-
 	for(j = 0; j < ndevs; j++)
 	{
 		handle->m_devs[j].m_buffer = (char*)MAP_FAILED;
@@ -153,7 +141,7 @@ scap_t* scap_open_live(char *error)
 	if((res = scap_proc_scan_proc_dir(handle, "/proc", -1, -1, NULL, error, true)) != SCAP_SUCCESS)
 	{
 		scap_close(handle);
-		snprintf(error, SCAP_LASTERR_SIZE, "error creating the process list");
+		snprintf(error, SCAP_LASTERR_SIZE, "error creating the process list. Make sure you have root credentials.");
 		return NULL;
 	}
 
@@ -176,17 +164,19 @@ scap_t* scap_open_live(char *error)
 
 		if((handle->m_devs[j].m_fd = open(dev, O_RDWR | O_SYNC)) < 0)
 		{
+			if(errno == EBUSY)
+			{
+				snprintf(error, SCAP_LASTERR_SIZE, "device %s is already open. You can't run multiple instances of sysdig.", dev);
+			}
+			else
+			{
+				snprintf(error, SCAP_LASTERR_SIZE, "error opening device %s. Make sure you have root credentials and that the sysdig-probe module is loaded.", dev);
+			}
+
 			scap_close(handle);
-			snprintf(error, SCAP_LASTERR_SIZE, "error opening device %s. Make sure you have root credentials and that the sysdig-probe module is loaded.", dev);
 			return NULL;
 		}
 
-		//
-		// Init the polling fd for the device
-		//
-		handle->m_pollfds[j].fd = handle->m_devs[j].m_fd;
-		handle->m_pollfds[j].events = POLLIN;
-
 		//
 		// Map the ring buffer
 		//
@@ -239,10 +229,10 @@ scap_t* scap_open_live(char *error)
 	}
 
 	return handle;
-#endif // _WIN32
+#endif // HAS_CAPTURE
 }
 
-scap_t* scap_open_offline(char* fname, char *error)
+scap_t* scap_open_offline(const char* fname, char *error)
 {
 	scap_t* handle = NULL;
 
@@ -262,12 +252,12 @@ scap_t* scap_open_offline(char* fname, char *error)
 	handle->m_devs = NULL;
 	handle->m_ndevs = 0;
 	handle->m_proclist = NULL;
-	handle->m_pollfds = NULL;
 	handle->m_evtcnt = 0;
 	handle->m_file = NULL;
 	handle->m_addrlist = NULL;
 	handle->m_userlist = NULL;
 	handle->m_machine_info.num_cpus = (uint32_t)-1;
+	handle->m_last_evt_dump_flags = 0;
 
 	handle->m_file_evt_buf = (char*)malloc(FILE_READ_BUF_SIZE);
 	if(!handle->m_file_evt_buf)
@@ -280,7 +270,7 @@ scap_t* scap_open_offline(char* fname, char *error)
 	//
 	// Open the file
 	//
-	handle->m_file = fopen(fname, "rb");
+	handle->m_file = gzopen(fname, "rb");
 	if(handle->m_file == NULL)
 	{
 		snprintf(error, SCAP_LASTERR_SIZE, "can't open file %s", fname);
@@ -323,11 +313,11 @@ void scap_close(scap_t* handle)
 {
 	if(handle->m_file)
 	{
-		fclose(handle->m_file);
+		gzclose(handle->m_file);
 	}
 	else
 	{
-#if !defined(_WIN32) && !defined(__APPLE__)
+#if defined(HAS_CAPTURE)
 		uint32_t j;
 
 		ASSERT(handle->m_file == NULL);
@@ -352,12 +342,7 @@ void scap_close(scap_t* handle)
 		{
 			free(handle->m_devs);
 		}
-
-		if(handle->m_pollfds != NULL)
-		{
-			free(handle->m_pollfds);
-		}
-#endif // _WIN32
+#endif // HAS_CAPTURE
 	}
 
 	if(handle->m_file_evt_buf)
@@ -415,8 +400,10 @@ uint32_t scap_get_ndevs(scap_t* handle)
 	return handle->m_ndevs;
 }
 
+#if defined(HAS_CAPTURE)
+
 #ifndef _WIN32
-inline void get_buf_pointers(struct ppm_ring_buffer_info* bufinfo, uint32_t* phead, uint32_t* ptail, uint32_t* pread_size)
+static inline void get_buf_pointers(struct ppm_ring_buffer_info* bufinfo, uint32_t* phead, uint32_t* ptail, uint32_t* pread_size)
 #else
 void get_buf_pointers(struct ppm_ring_buffer_info* bufinfo, uint32_t* phead, uint32_t* ptail, uint32_t* pread_size)
 #endif
@@ -434,7 +421,6 @@ void get_buf_pointers(struct ppm_ring_buffer_info* bufinfo, uint32_t* phead, uin
 	}
 }
 
-#if !defined(_WIN32) && !defined(__APPLE__)
 int32_t scap_readbuf(scap_t* handle, uint32_t cpuid, bool blocking, OUT char** buf, OUT uint32_t* len)
 {
 	uint32_t thead;
@@ -564,7 +550,7 @@ bool check_scap_next_wait(scap_t* handle)
 	return true;
 }
 
-#endif // _WIN32
+#endif // HAS_CAPTURE
 
 #ifndef _WIN32
 static inline int32_t scap_next_live(scap_t* handle, OUT scap_evt** pevent, OUT uint16_t* pcpuid)
@@ -572,7 +558,7 @@ static inline int32_t scap_next_live(scap_t* handle, OUT scap_evt** pevent, OUT
 static int32_t scap_next_live(scap_t* handle, OUT scap_evt** pevent, OUT uint16_t* pcpuid)
 #endif
 {
-#if defined(_WIN32) || defined(__APPLE__)
+#if !defined(HAS_CAPTURE)
 	//
 	// this should be prevented at open time
 	//
@@ -580,7 +566,7 @@ static int32_t scap_next_live(scap_t* handle, OUT scap_evt** pevent, OUT uint16_
 	return SCAP_FAILURE;
 #else
 	uint32_t j;
-	uint64_t max_ts = 0xffffffffffffffff;
+	uint64_t max_ts = 0xffffffffffffffffLL;
 	uint64_t max_buf_size = 0;
 	scap_evt* pe = NULL;
 	bool waited = false;
@@ -739,11 +725,8 @@ int32_t scap_get_stats(scap_t* handle, OUT scap_stats* stats)
 //
 int32_t scap_stop_capture(scap_t* handle)
 {
-#ifdef _WIN32
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on windows");
-	return SCAP_FAILURE;
-#elif defined(__APPLE__)
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on OSX");
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on %s", PLATFORM_NAME);
 	return SCAP_FAILURE;
 #else
 	uint32_t j;
@@ -778,7 +761,7 @@ int32_t scap_stop_capture(scap_t* handle)
 	handle->m_emptybuf_timeout_ms = 0;
 
 	return SCAP_SUCCESS;
-#endif // _WIN32
+#endif // HAS_CAPTURE
 }
 
 //
@@ -786,11 +769,8 @@ int32_t scap_stop_capture(scap_t* handle)
 //
 int32_t scap_start_capture(scap_t* handle)
 {
-#ifdef _WIN32
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture non supported on windows");
-	return SCAP_FAILURE;
-#elif defined(__APPLE__)
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture non supported on OSX");
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on %s", PLATFORM_NAME);
 	return SCAP_FAILURE;
 #else
 	uint32_t j;
@@ -819,10 +799,10 @@ int32_t scap_start_capture(scap_t* handle)
 	}
 
 	return SCAP_SUCCESS;
-#endif // _WIN32
+#endif // HAS_CAPTURE
 }
 
-#if !defined (_WIN32) && !defined(__APPLE__)
+#if defined(HAS_CAPTURE)
 static int32_t scap_set_dropping_mode(scap_t* handle, int request, uint32_t sampling_ratio)
 {
 	//	
@@ -851,11 +831,8 @@ static int32_t scap_set_dropping_mode(scap_t* handle, int request, uint32_t samp
 
 int32_t scap_stop_dropping_mode(scap_t* handle)
 {
-#ifdef _WIN32
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on windows");
-	return SCAP_FAILURE;
-#elif defined(__APPLE__)
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on OSX");
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on %s", PLATFORM_NAME);
 	return SCAP_FAILURE;
 #else
 	return scap_set_dropping_mode(handle, PPM_IOCTL_DISABLE_DROPPING_MODE, 0);
@@ -864,11 +841,8 @@ int32_t scap_stop_dropping_mode(scap_t* handle)
 
 int32_t scap_start_dropping_mode(scap_t* handle, uint32_t sampling_ratio)
 {
-#ifdef _WIN32
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on windows");
-	return SCAP_FAILURE;
-#elif defined(__APPLE__)
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on OSX");
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on %s", PLATFORM_NAME);
 	return SCAP_FAILURE;
 #else
 	return scap_set_dropping_mode(handle, PPM_IOCTL_ENABLE_DROPPING_MODE, sampling_ratio);
@@ -920,11 +894,8 @@ int32_t scap_set_snaplen(scap_t* handle, uint32_t snaplen)
 		return SCAP_FAILURE;
 	}
 
-#ifdef _WIN32
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on windows");
-	return SCAP_FAILURE;
-#elif defined(__APPLE__)
-	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on OSX");
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "live capture not supported on %s", PLATFORM_NAME);
 	return SCAP_FAILURE;
 #else
 	//
@@ -959,3 +930,106 @@ int32_t scap_set_snaplen(scap_t* handle, uint32_t snaplen)
 #endif
 }
 
+int64_t scap_get_readfile_offset(scap_t* handle)
+{
+	if(handle->m_file == NULL)
+	{
+		snprintf(handle->m_lasterr,	SCAP_LASTERR_SIZE, "scap_get_readfile_offset only works on trace files");
+		return -1;
+	}
+
+	return gzoffset(handle->m_file);
+}
+
+static int32_t scap_handle_eventmask(scap_t* handle, uint32_t op, uint32_t event_id)
+{
+	//
+	// Not supported on files
+	//
+	if(handle->m_file)
+	{
+		snprintf(handle->m_lasterr,	SCAP_LASTERR_SIZE, "manipulating eventmasks not supported on offline captures");
+		return SCAP_FAILURE;
+	}
+
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "eventmask not supported on %s", PLATFORM_NAME);
+	return SCAP_FAILURE;
+#else
+	//
+	// Tell the driver to change the snaplen
+	//
+
+	switch(op) {
+	case PPM_IOCTL_MASK_ZERO_EVENTS:
+	case PPM_IOCTL_MASK_SET_EVENT:
+	case PPM_IOCTL_MASK_UNSET_EVENT:
+		break;
+
+	default:
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "%s(%d) internal error", __FUNCTION__, op);
+		ASSERT(false);
+		return SCAP_FAILURE;
+		break;
+	}
+
+	if(ioctl(handle->m_devs[0].m_fd, op, event_id))
+	{
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "%s(%d) failed", __FUNCTION__, op);
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+	{
+		uint32_t j;
+
+		//
+		// Force a flush of the read buffers, so we don't capture events with the old snaplen
+		//
+		for(j = 0; j < handle->m_ndevs; j++)
+		{
+			scap_readbuf(handle,
+				j,
+				false,
+				&handle->m_devs[j].m_sn_next_event,
+				&handle->m_devs[j].m_sn_len);
+
+			handle->m_devs[j].m_sn_len = 0;
+		}
+	}
+
+	return SCAP_SUCCESS;
+#endif
+}
+
+int32_t scap_clear_eventmask(scap_t* handle) {
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "eventmask not supported on %s", PLATFORM_NAME);
+	return SCAP_FAILURE;
+#else
+	return(scap_handle_eventmask(handle, PPM_IOCTL_MASK_ZERO_EVENTS, 0));
+#endif
+}
+
+int32_t scap_set_eventmask(scap_t* handle, uint32_t event_id) {
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "eventmask not supported on %s", PLATFORM_NAME);
+	return SCAP_FAILURE;
+#else
+	return(scap_handle_eventmask(handle, PPM_IOCTL_MASK_SET_EVENT, event_id));
+#endif
+}
+
+int32_t scap_unset_eventmask(scap_t* handle, uint32_t event_id) {
+#if !defined(HAS_CAPTURE)
+	snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "eventmask not supported on %s", PLATFORM_NAME);
+	return SCAP_FAILURE;
+#else
+	return(scap_handle_eventmask(handle, PPM_IOCTL_MASK_UNSET_EVENT, event_id));
+#endif
+}
+
+uint32_t scap_event_get_dump_flags(scap_t* handle)
+{
+	return handle->m_last_evt_dump_flags;
+}
