@@ -22,7 +22,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 // to configure everything with crappy documentation and code that doesn't compile,
 // I decided that I agree with this http://mortoray.com/2012/07/20/why-i-dont-use-a-parser-generator/
 // and that I'm going with a manually written parser. The grammar is simple enough that it's not
-// going to take more time. On the other hand I will avoid a crappy dependency that breaks my 
+// going to take more time. On the other hand I will avoid a crappy dependency that breaks my
 // code at every new release, and I will have a cleaner and easier to understand code base.
 //
 
@@ -33,6 +33,13 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "filter.h"
 #include "filterchecks.h"
 
+#ifndef _GNU_SOURCE
+//
+// Fallback implementation of memmem
+//
+void *memmem(const void *haystack, size_t haystacklen, const void *needle, size_t needlelen);
+#endif
+
 extern sinsp_filter_check_list g_filterlist;
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -42,12 +49,13 @@ sinsp_filter_check_list::sinsp_filter_check_list()
 {
 	//////////////////////////////////////////////////////////////////////////////
 	// ADD NEW FILTER CHECK CLASSES HERE
-	//////////////////////////////////////////////////////////////////////////////	
+	//////////////////////////////////////////////////////////////////////////////
 	add_filter_check(new sinsp_filter_check_fd());
 	add_filter_check(new sinsp_filter_check_thread());
 	add_filter_check(new sinsp_filter_check_event());
 	add_filter_check(new sinsp_filter_check_user());
 	add_filter_check(new sinsp_filter_check_group());
+	add_filter_check(new sinsp_filter_check_syslog());
 }
 
 sinsp_filter_check_list::~sinsp_filter_check_list()
@@ -75,7 +83,7 @@ void sinsp_filter_check_list::get_all_fields(OUT vector<const filter_check_info*
 	}
 }
 
-sinsp_filter_check* sinsp_filter_check_list::new_filter_check_from_fldname(string name, 
+sinsp_filter_check* sinsp_filter_check_list::new_filter_check_from_fldname(const string& name,
 																		   sinsp* inspector,
 																		   bool do_exact_check)
 {
@@ -110,7 +118,6 @@ field_not_found:
 	// it's very likely that you've forgotten to add your filter to the list in
 	// the constructor
 	//
-	ASSERT(false);
 	return NULL;
 }
 
@@ -172,13 +179,13 @@ bool flt_compare_string(ppm_cmp_operator op, char* operand1, char* operand2)
 	case CO_CONTAINS:
 		return (strstr(operand1, operand2) != NULL);
 	case CO_LT:
-		throw sinsp_exception("'<' not supported for numeric filters");
+		throw sinsp_exception("'<' not supported for string filters");
 	case CO_LE:
-		throw sinsp_exception("'<=' not supported for numeric filters");
+		throw sinsp_exception("'<=' not supported for string filters");
 	case CO_GT:
-		throw sinsp_exception("'>' not supported for numeric filters");
+		throw sinsp_exception("'>' not supported for string filters");
 	case CO_GE:
-		throw sinsp_exception("'>=' not supported for numeric filters");
+		throw sinsp_exception("'>=' not supported for string filters");
 	default:
 		ASSERT(false);
 		throw sinsp_exception("invalid filter operator " + std::to_string((long long) op));
@@ -186,7 +193,32 @@ bool flt_compare_string(ppm_cmp_operator op, char* operand1, char* operand2)
 	}
 }
 
-bool flt_compare(ppm_cmp_operator op, ppm_param_type type, void* operand1, void* operand2)
+bool flt_compare_buffer(ppm_cmp_operator op, char* operand1, char* operand2, uint32_t op1_len, uint32_t op2_len)
+{
+	switch(op)
+	{
+	case CO_EQ:
+		return op1_len == op2_len && (memcmp(operand1, operand2, op1_len) == 0);
+	case CO_NE:
+		return op1_len != op2_len || (memcmp(operand1, operand2, op1_len) != 0);
+	case CO_CONTAINS:
+		return (memmem(operand1, op1_len, operand2, op2_len) != NULL);
+	case CO_LT:
+		throw sinsp_exception("'<' not supported for buffer filters");
+	case CO_LE:
+		throw sinsp_exception("'<=' not supported for buffer filters");
+	case CO_GT:
+		throw sinsp_exception("'>' not supported for buffer filters");
+	case CO_GE:
+		throw sinsp_exception("'>=' not supported for buffer filters");
+	default:
+		ASSERT(false);
+		throw sinsp_exception("invalid filter operator " + std::to_string((long long) op));
+		return false;
+	}
+}
+
+bool flt_compare(ppm_cmp_operator op, ppm_param_type type, void* operand1, void* operand2, uint32_t op1_len, uint32_t op2_len)
 {
 	switch(type)
 	{
@@ -222,7 +254,7 @@ bool flt_compare(ppm_cmp_operator op, ppm_param_type type, void* operand1, void*
 	case PT_CHARBUF:
 		return flt_compare_string(op, (char*)operand1, (char*)operand2);
 	case PT_BYTEBUF:
-		throw sinsp_exception("bytebuf comparison not implemented yet");
+		return flt_compare_buffer(op, (char*)operand1, (char*)operand2, op1_len, op2_len);
 	case PT_SOCKADDR:
 	case PT_SOCKTUPLE:
 	case PT_FDLIST:
@@ -245,6 +277,7 @@ sinsp_filter_check::sinsp_filter_check() :
 	m_field = NULL;
 	m_info.m_fields = NULL;
 	m_info.m_nfiedls = -1;
+	m_val_storage_len = 0;
 }
 
 void sinsp_filter_check::set_inspector(sinsp* inspector)
@@ -252,6 +285,154 @@ void sinsp_filter_check::set_inspector(sinsp* inspector)
 	m_inspector = inspector;
 }
 
+Json::Value sinsp_filter_check::rawval_to_json(uint8_t* rawval, const filtercheck_field_info* finfo, uint32_t len)
+{
+	ASSERT(rawval != NULL);
+	ASSERT(finfo != NULL);
+
+	switch(finfo->m_type)
+	{
+		case PT_INT8:
+			if(finfo->m_print_format == PF_DEC)
+			{
+				return *(int8_t *)rawval;
+			}
+			else if(finfo->m_print_format == PF_HEX)
+			{
+				return rawval_to_string(rawval, finfo, len);
+			}
+			else
+			{
+				ASSERT(false);
+				return Json::Value::null;
+			}
+
+		case PT_INT16:
+			if(finfo->m_print_format == PF_DEC)
+			{
+				return *(int16_t *)rawval;
+			}
+			else if(finfo->m_print_format == PF_HEX)
+			{
+				return rawval_to_string(rawval, finfo, len);
+			}
+			else
+			{
+				ASSERT(false);
+				return Json::Value::null;
+			}
+
+		case PT_INT32:
+			if(finfo->m_print_format == PF_DEC)
+			{
+				return *(int32_t *)rawval;
+			}
+			else if(finfo->m_print_format == PF_HEX)
+			{
+				return rawval_to_string(rawval, finfo, len);
+			}
+			else
+			{
+				ASSERT(false);
+				return Json::Value::null;
+			}
+
+		case PT_INT64:
+		case PT_PID:
+			if(finfo->m_print_format == PF_DEC)
+			{
+		 		return (Json::Value::Int64)*(int64_t *)rawval;
+			}
+			else 
+			{
+				return rawval_to_string(rawval, finfo, len);
+			}
+
+		case PT_L4PROTO: // This can be resolved in the future
+		case PT_UINT8:
+			if(finfo->m_print_format == PF_DEC)
+			{
+				return *(uint8_t *)rawval;
+			}
+			else if(finfo->m_print_format == PF_HEX)
+			{
+				return rawval_to_string(rawval, finfo, len);
+			}
+			else
+			{
+				ASSERT(false);
+				return Json::Value::null;
+			}
+
+		case PT_PORT: // This can be resolved in the future
+		case PT_UINT16:
+			if(finfo->m_print_format == PF_DEC)
+			{
+				return *(uint16_t *)rawval;
+			}
+			else if(finfo->m_print_format == PF_HEX)
+			{
+				return rawval_to_string(rawval, finfo, len);
+			}
+			else
+			{
+				ASSERT(false);
+				return Json::Value::null;
+			}
+
+		case PT_UINT32:
+			if(finfo->m_print_format == PF_DEC)
+			{
+				return *(uint32_t *)rawval;
+			}
+			else if(finfo->m_print_format == PF_HEX)
+			{
+				return rawval_to_string(rawval, finfo, len);
+			}
+			else
+			{
+				ASSERT(false);
+				return Json::Value::null;
+			}
+
+		case PT_UINT64:
+		case PT_RELTIME:
+		case PT_ABSTIME:
+			if(finfo->m_print_format == PF_DEC)
+			{
+				return (Json::Value::UInt64)*(uint64_t *)rawval;
+			}
+			else if(
+				finfo->m_print_format == PF_10_PADDED_DEC ||
+				finfo->m_print_format == PF_HEX)
+			{
+				return rawval_to_string(rawval, finfo, len);
+			}
+			else
+			{
+				ASSERT(false);
+				return Json::Value::null;
+			}
+
+		case PT_SOCKADDR:
+		case PT_SOCKFAMILY:
+			ASSERT(false);
+			return Json::Value::null;
+
+		case PT_BOOL:
+			return Json::Value((bool)(*(uint32_t*)rawval != 0));
+
+		case PT_CHARBUF:
+		case PT_BYTEBUF:
+		case PT_IPV4ADDR:
+			return rawval_to_string(rawval, finfo, len);
+
+		default:
+			ASSERT(false);
+			throw sinsp_exception("wrong event type " + to_string((long long) finfo->m_type));
+	}
+}
+
 char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_field_info* finfo, uint32_t len)
 {
 	char* prfmt;
@@ -320,6 +501,7 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 			return m_getpropertystr_storage;
 		case PT_INT64:
 		case PT_PID:
+		case PT_ERRNO:
 			if(finfo->m_print_format == PF_DEC)
 			{
 				prfmt = (char*)"%" PRId64;
@@ -475,7 +657,7 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 	}
 }
 
-void sinsp_filter_check::string_to_rawval(const char* str, ppm_param_type ptype)
+void sinsp_filter_check::string_to_rawval(const char* str, uint32_t len, ppm_param_type ptype)
 {
 	switch(ptype)
 	{
@@ -489,6 +671,7 @@ void sinsp_filter_check::string_to_rawval(const char* str, ppm_param_type ptype)
 			*(int32_t*)(&m_val_storage[0]) = sinsp_numparser::parsed32(str);
 			break;
 		case PT_INT64:
+		case PT_FD:
 		case PT_ERRNO:
 			*(int64_t*)(&m_val_storage[0]) = sinsp_numparser::parsed64(str);
 			break;
@@ -517,7 +700,7 @@ void sinsp_filter_check::string_to_rawval(const char* str, ppm_param_type ptype)
 		case PT_SOCKADDR:
 		case PT_SOCKFAMILY:
 			{
-				uint32_t len = strlen(str);
+				len = (uint32_t)strlen(str);
 				if(len >= m_val_storage.size())
 				{
 					throw sinsp_exception("filter parameter too long:" + string(str));
@@ -548,6 +731,15 @@ void sinsp_filter_check::string_to_rawval(const char* str, ppm_param_type ptype)
 				throw sinsp_exception("unrecognized IP address " + string(str));
 			}
 			break;
+		case PT_BYTEBUF:
+			if(len >= m_val_storage.size())
+			{
+				throw sinsp_exception("filter parameter too long:" + string(str));
+			}
+
+			memcpy((&m_val_storage[0]), str, len);
+			m_val_storage_len = len;
+			break;
 		default:
 			ASSERT(false);
 			throw sinsp_exception("wrong event type " + to_string((long long) m_field->m_type));
@@ -567,6 +759,24 @@ char* sinsp_filter_check::tostring(sinsp_evt* evt)
 	return rawval_to_string(rawval, m_field, len);
 }
 
+Json::Value sinsp_filter_check::tojson(sinsp_evt* evt)
+{
+	uint32_t len;
+	Json::Value jsonval = extract_as_js(evt, &len);
+
+	if(jsonval == Json::Value::null) 
+	{
+		uint8_t* rawval = extract(evt, &len);
+		if(rawval == NULL)
+		{
+			return Json::Value::null;
+		}
+		return rawval_to_json(rawval, m_field, len);
+	} 
+	
+	return jsonval;
+}
+
 int32_t sinsp_filter_check::parse_field_name(const char* str)
 {
 	int32_t j;
@@ -582,7 +792,7 @@ int32_t sinsp_filter_check::parse_field_name(const char* str)
 	for(j = 0; j < m_info.m_nfiedls; j++)
 	{
 		string fldname = m_info.m_fields[j].m_name;
-		int32_t fldlen = fldname.length();
+		int32_t fldlen = (uint32_t)fldname.length();
 
 		if(val.compare(0, fldlen, fldname) == 0)
 		{
@@ -598,9 +808,9 @@ int32_t sinsp_filter_check::parse_field_name(const char* str)
 	return max_fldlen;
 }
 
-void sinsp_filter_check::parse_filter_value(const char* str)
+void sinsp_filter_check::parse_filter_value(const char* str, uint32_t len)
 {
-	string_to_rawval(str, m_field->m_type);
+	string_to_rawval(str, len, m_field->m_type);
 }
 
 const filtercheck_field_info* sinsp_filter_check::get_field_info()
@@ -618,10 +828,12 @@ bool sinsp_filter_check::compare(sinsp_evt *evt)
 		return false;
 	}
 
-	return flt_compare(m_cmpop, 
-		m_info.m_fields[m_field_id].m_type, 
-		extracted_val, 
-		&m_val_storage[0]);
+	return flt_compare(m_cmpop,
+		m_info.m_fields[m_field_id].m_type,
+		extracted_val,
+		&m_val_storage[0],
+		len,
+		m_val_storage_len);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -660,10 +872,10 @@ void sinsp_filter_expression::parse(string expr)
 bool sinsp_filter_expression::compare(sinsp_evt *evt)
 {
 	uint32_t j;
-	uint32_t size = m_checks.size();
+	uint32_t size = (uint32_t)m_checks.size();
 	bool res = true;
 	bool chkres;
-	 
+
 	for(j = 0; j < size; j++)
 	{
 		sinsp_filter_check* chk = m_checks[j];
@@ -714,7 +926,7 @@ bool sinsp_filter_expression::compare(sinsp_evt *evt)
 ///////////////////////////////////////////////////////////////////////////////
 // sinsp_filter implementation
 ///////////////////////////////////////////////////////////////////////////////
-sinsp_filter::sinsp_filter(sinsp* inspector, string fltstr)
+sinsp_filter::sinsp_filter(sinsp* inspector, const string& fltstr)
 {
 	m_inspector = inspector;
 	m_scanpos = -1;
@@ -799,9 +1011,20 @@ char sinsp_filter::next()
 	}
 }
 
-string sinsp_filter::next_operand(bool expecting_first_operand)
+vector<char> sinsp_filter::next_operand(bool expecting_first_operand)
 {
+	vector<char> res;
+	bool is_quoted = false;
 	int32_t start;
+	int32_t nums[2];
+	uint32_t num_pos;
+	enum ppm_escape_state
+	{
+		PES_NORMAL,
+		PES_SLASH,
+		PES_NUMBER,
+		PES_ERROR,
+	} escape_state;
 
 	//
 	// Skip spaces
@@ -811,15 +1034,26 @@ string sinsp_filter::next_operand(bool expecting_first_operand)
 		next();
 	}
 
+	//
+	// If there are quotes, don't stop on blank
+	//
+	if(m_scanpos < m_scansize && 
+		(m_fltstr[m_scanpos] == '"' || m_fltstr[m_scanpos] == '\''))
+	{
+		is_quoted = true;
+		m_scanpos++;
+	}
+
 	//
 	// Mark the beginning of the word
 	//
 	start = m_scanpos;
+	escape_state = PES_NORMAL;
+	num_pos = 0;
 
-	for(; m_scanpos < m_scansize; m_scanpos++)
+	while(m_scanpos < m_scansize && escape_state != PES_ERROR)
 	{
 		char curchar = m_fltstr[m_scanpos];
-		
 		bool is_end_of_word;
 
 		if(expecting_first_operand)
@@ -828,33 +1062,108 @@ string sinsp_filter::next_operand(bool expecting_first_operand)
 		}
 		else
 		{
-			is_end_of_word = (isblank(curchar) || is_bracket(curchar));
+			is_end_of_word = (!is_quoted && (isblank(curchar) || is_bracket(curchar))) ||
+				(is_quoted && escape_state != PES_SLASH && (curchar == '"' || curchar == '\''));
 		}
 
 		if(is_end_of_word)
 		{
+			if(escape_state != PES_NORMAL)
+			{
+				escape_state = PES_ERROR;
+				break;
+			}
+
 			//
 			// End of word
 			//
 			ASSERT(m_scanpos > start);
-			string res = m_fltstr.substr(start, m_scanpos - start);
 
 			if(curchar == '(' || curchar == ')')
 			{
 				m_scanpos--;
 			}
 
+			res.push_back('\0');
 			return res;
 		}
+
+		switch(escape_state)
+		{
+		case PES_NORMAL:
+			if(curchar == '\\' && !expecting_first_operand)
+			{
+				escape_state = PES_SLASH;
+			}
+			else
+			{
+				res.push_back(curchar);
+			}
+			break;
+		case PES_SLASH:
+			switch(curchar)
+			{
+			case '\\':
+			case '"':
+				escape_state = PES_NORMAL;
+				res.push_back(curchar);
+				break;
+			case 'x':
+				escape_state = PES_NUMBER;
+				break;
+			default:
+				escape_state = PES_ERROR;
+				break;
+			}
+			break;
+		case PES_NUMBER:
+			if(isdigit((int)curchar))
+			{
+				nums[num_pos++] = curchar - '0';
+			}
+			else if((curchar >= 'a' && curchar <= 'f') || (curchar >= 'A' && curchar <= 'F'))
+			{
+				nums[num_pos++] = tolower((int)curchar) - 'a' + 10;
+			}
+			else
+			{
+				escape_state = PES_ERROR;
+			}
+
+			if(num_pos == 2 && escape_state != PES_ERROR)
+			{
+				res.push_back((char)(nums[0] * 16 + nums[1]));
+
+				num_pos = 0;
+				escape_state = PES_NORMAL;
+			}
+			break;
+		default:
+			ASSERT(false);
+			escape_state = PES_ERROR;
+			break;
+		}
+
+		m_scanpos++;
+	}
+
+	if(escape_state == PES_ERROR)
+	{
+		throw sinsp_exception("filter error: unrecognized escape sequence at " + m_fltstr.substr(start, m_scanpos));
+	}
+	else if(is_quoted)
+	{
+		throw sinsp_exception("filter error: unclosed quotes");
 	}
 
 	//
 	// End of filter
 	//
-	return m_fltstr.substr(start, m_scansize - 1);
+	res.push_back('\0');
+	return res;
 }
 
-bool sinsp_filter::compare_no_consume(string str)
+bool sinsp_filter::compare_no_consume(const string& str)
 {
 	if(m_scanpos + (int32_t)str.size() >= m_scansize)
 	{
@@ -934,22 +1243,23 @@ ppm_cmp_operator sinsp_filter::next_comparison_operator()
 void sinsp_filter::parse_check(sinsp_filter_expression* parent_expr, boolop op)
 {
 	uint32_t startpos = m_scanpos;
-	string operand1 = next_operand(true);
-	sinsp_filter_check* chk = g_filterlist.new_filter_check_from_fldname(operand1, m_inspector, true);
+	vector<char> operand1 = next_operand(true);
+	string str_operand1 = string((char *)&operand1[0]);
+	sinsp_filter_check* chk = g_filterlist.new_filter_check_from_fldname(str_operand1, m_inspector, true);
 
 	if(chk == NULL)
 	{
-		throw sinsp_exception("filter error: unrecognized field " + 
-			operand1 + " at pos " + to_string((long long) startpos));
+		throw sinsp_exception("filter error: unrecognized field " +
+			str_operand1 + " at pos " + to_string((long long) startpos));
 	}
 
 	ppm_cmp_operator co = next_comparison_operator();
-	string operand2 = next_operand(false);
+	vector<char> operand2 = next_operand(false);
 
 	chk->m_boolop = op;
 	chk->m_cmpop = co;
-	chk->parse_field_name(operand1.c_str());
-	chk->parse_filter_value(operand2.c_str());
+	chk->parse_field_name((char *)&operand1[0]);
+	chk->parse_filter_value((char *)&operand2[0], (uint32_t)operand2.size() - 1);
 
 	parent_expr->add_check(chk);
 }
@@ -974,10 +1284,10 @@ void sinsp_filter::pop_expression()
 	m_nest_level--;
 }
 
-void sinsp_filter::compile(string fltstr)
+void sinsp_filter::compile(const string& fltstr)
 {
 	m_fltstr = fltstr;
-	m_scansize = m_fltstr.size();
+	m_scansize = (uint32_t)m_fltstr.size();
 
 	while(true)
 	{
