@@ -23,6 +23,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <algorithm>
 #include "sinsp.h"
 #include "sinsp_int.h"
+#include "protodecoder.h"
 
 static void copy_ipv6_address(uint32_t* dest, uint32_t* src)
 {
@@ -63,7 +64,11 @@ void sinsp_threadinfo::init()
 	m_flags = PPM_CL_NAME_CHANGED;
 	m_nchilds = 0;
 	m_fdlimit = -1;
-	m_fd_usage_pct = 0;
+	m_vmsize_kb = 0;
+	m_vmrss_kb = 0;
+	m_vmswap_kb = 0;
+	m_pfmajor = 0;
+	m_pfminor = 0;
 	m_main_thread = NULL;
 	m_main_program_thread = NULL;
 	m_lastevent_fd = 0;
@@ -156,12 +161,18 @@ void sinsp_threadinfo::init(const scap_threadinfo* pi)
 
 	m_exe = pi->exe;
 	set_args(pi->args, pi->args_len);
-	set_cwd(pi->cwd, strlen(pi->cwd));
+	set_cwd(pi->cwd, (uint32_t)strlen(pi->cwd));
 	m_flags |= pi->flags;
 	m_fdtable.clear();
 	m_fdlimit = pi->fdlimit;
 	m_uid = pi->uid;
 	m_gid = pi->gid;
+	m_vmsize_kb = pi->vmsize_kb;
+	m_vmrss_kb = pi->vmrss_kb;
+	m_vmswap_kb = pi->vmswap_kb;
+	m_pfmajor = pi->pfmajor;
+	m_pfminor = pi->pfminor;
+	m_nchilds = 0;
 
 	HASH_ITER(hh, pi->fdlist, fdi, tfdi)
 	{
@@ -172,6 +183,8 @@ void sinsp_threadinfo::init(const scap_threadinfo* pi)
 		newfdi.m_type = fdi->type;
 		newfdi.m_flags = sinsp_fdinfo_t::FLAGS_FROM_PROC;
 		newfdi.m_ino = fdi->ino;
+		newfdi.m_write_callbacks.clear();
+		newfdi.m_read_callbacks.clear();
 
 		switch(newfdi.m_type)
 		{
@@ -267,6 +280,21 @@ void sinsp_threadinfo::init(const scap_threadinfo* pi)
 			break;
 		}
 
+		//
+		// Call the protocol decoder callbacks associated to notify them about this FD
+		//
+		ASSERT(m_inspector != NULL);
+		vector<sinsp_protodecoder*>::iterator it;
+
+		for(it = m_inspector->m_parser->m_open_callbacks.begin(); 
+			it != m_inspector->m_parser->m_open_callbacks.end(); ++it)
+		{
+			(*it)->on_fd_from_proc(&newfdi);
+		}
+
+		//
+		// Add the FD to the table
+		//
 		if(do_add)
 		{
 			m_fdtable.add(fdi->fd, &newfdi);
@@ -324,7 +352,7 @@ sinsp_threadinfo* sinsp_threadinfo::get_main_thread()
 			//
 			// Yes, this is a child thread. Find the process root thread.
 			//
-			sinsp_threadinfo *ptinfo = m_inspector->get_thread(m_pid, true);
+			sinsp_threadinfo *ptinfo = m_inspector->get_thread(m_pid, true, true);
 			if(NULL == ptinfo)
 			{
 				ASSERT(false);
@@ -340,7 +368,7 @@ sinsp_threadinfo* sinsp_threadinfo::get_main_thread()
 
 sinsp_threadinfo* sinsp_threadinfo::get_parent_thread()
 {
-	return m_inspector->get_thread(m_ptid, false);
+	return m_inspector->get_thread(m_ptid, false, true);
 }
 
 sinsp_fdtable* sinsp_threadinfo::get_fd_table()
@@ -408,8 +436,7 @@ bool sinsp_threadinfo::is_bound_to_port(uint16_t number)
 
 	sinsp_fdtable* fdt = get_fd_table();
 
-	for(it = fdt->m_table.begin(); 
-		it != fdt->m_table.end(); ++it)
+	for(it = fdt->m_table.begin(); it != fdt->m_table.end(); ++it)
 	{
 		if(it->second.m_type == SCAP_FD_IPV4_SOCK)
 		{
@@ -527,7 +554,7 @@ void sinsp_threadinfo::set_cwd(const char* cwd, uint32_t cwdlen)
 		sinsp_utils::concatenate_paths(tpath, 
 			SCAP_MAX_PATH_SIZE, 
 			(char*)tinfo->m_cwd.c_str(), 
-			tinfo->m_cwd.size(), 
+			(uint32_t)tinfo->m_cwd.size(), 
 			cwd, 
 			cwdlen);
 
@@ -574,6 +601,37 @@ void* sinsp_threadinfo::get_private_state(uint32_t id)
 	return m_private_state[id];
 }
 
+uint64_t sinsp_threadinfo::get_fd_usage_pct()
+{
+	int64_t fdlimit = get_fd_limit();
+	if(fdlimit > 0)
+	{
+		uint64_t fd_opencount = get_fd_opencount();
+		ASSERT(fd_opencount <= (uint64_t) fdlimit);
+		if(fd_opencount <= (uint64_t) fdlimit)
+		{
+			return (fd_opencount * 100) / fdlimit;
+		}
+		else
+		{
+			return 100;
+		}
+	}
+	else
+	{
+		return 0;
+	}
+}
+
+uint64_t sinsp_threadinfo::get_fd_opencount()
+{
+	return get_main_thread()->m_fdtable.size();
+}
+
+uint64_t sinsp_threadinfo::get_fd_limit()
+{
+	return get_main_thread()->m_fdlimit;
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 // sinsp_thread_manager implementation
@@ -607,7 +665,7 @@ void sinsp_thread_manager::set_listener(sinsp_threadtable_listener* listener)
 	m_listener = listener;
 }
 
-sinsp_threadinfo* sinsp_thread_manager::get_thread(int64_t tid)
+sinsp_threadinfo* sinsp_thread_manager::get_thread(int64_t tid, bool lookup_only)
 {
 	threadinfo_map_iterator_t it;
 
@@ -633,9 +691,12 @@ sinsp_threadinfo* sinsp_thread_manager::get_thread(int64_t tid)
 #ifdef GATHER_INTERNAL_STATS
 		m_non_cached_lookups->increment();
 #endif
-		m_last_tid = tid;
-		m_last_tinfo = &(it->second);
-		m_last_tinfo->m_lastaccess_ts = m_inspector->m_lastevent_ts;
+		if(!lookup_only)
+		{
+			m_last_tid = tid;
+			m_last_tinfo = &(it->second);
+			m_last_tinfo->m_lastaccess_ts = m_inspector->m_lastevent_ts;
+		}
 		return &(it->second);
 	}
 	else
@@ -656,7 +717,8 @@ void sinsp_thread_manager::increment_mainthread_childcount(sinsp_threadinfo* thr
 		// be deleted (if it calls pthread_exit()) until we are done
 		//
 		ASSERT(threadinfo->m_pid != threadinfo->m_tid);
-		sinsp_threadinfo* main_thread = m_inspector->get_thread(threadinfo->m_pid, false);
+
+		sinsp_threadinfo* main_thread = m_inspector->get_thread(threadinfo->m_pid, true, true);
 		if(main_thread)
 		{
 			++main_thread->m_nchilds;
@@ -668,20 +730,31 @@ void sinsp_thread_manager::increment_mainthread_childcount(sinsp_threadinfo* thr
 	}
 }
 
-void sinsp_thread_manager::increment_program_childcount(sinsp_threadinfo* threadinfo)
+void sinsp_thread_manager::increment_program_childcount(sinsp_threadinfo* threadinfo, uint32_t level, uint32_t notclosed_level)
 {
 	if(threadinfo->is_main_thread())
 	{
-		sinsp_threadinfo* parent_thread = m_inspector->get_thread(threadinfo->m_ptid, false);
+		sinsp_threadinfo* parent_thread = m_inspector->get_thread(threadinfo->m_ptid, false, false);
 
 		if(parent_thread)
 		{
+			if(parent_thread->m_tid == threadinfo->m_tid || level > 64)
+			{
+				return;
+			}
+			
 			if((parent_thread->m_comm == threadinfo->m_comm) &&
 				(parent_thread->m_exe == threadinfo->m_exe))
 			{
 				threadinfo->m_progid = parent_thread->m_tid;
 				++parent_thread->m_nchilds;
-				increment_program_childcount(parent_thread);
+
+				if(!(threadinfo->m_flags & PPM_CL_CLOSED))
+				{
+					notclosed_level++;
+				}
+
+				increment_program_childcount(parent_thread, level + 1, notclosed_level);
 			}
 		}
 	}
@@ -694,7 +767,7 @@ void sinsp_thread_manager::decrement_program_childcount(sinsp_threadinfo* thread
 	{
 		ASSERT(threadinfo->m_pid != threadinfo->m_progid);
 
-		sinsp_threadinfo* prog_thread = m_inspector->get_thread(threadinfo->m_progid, false);
+		sinsp_threadinfo* prog_thread = m_inspector->get_thread(threadinfo->m_progid, false, true);
 
 		if(prog_thread)
 		{
@@ -702,6 +775,7 @@ void sinsp_thread_manager::decrement_program_childcount(sinsp_threadinfo* thread
 			{
 				--prog_thread->m_nchilds;
 				decrement_program_childcount(prog_thread, level + 1);
+				threadinfo->m_main_program_thread = NULL;
 			}
 			else
 			{
@@ -732,7 +806,7 @@ void sinsp_thread_manager::add_thread(sinsp_threadinfo& threadinfo, bool from_sc
 	if(!from_scap_proctable)
 	{
 		increment_mainthread_childcount(&threadinfo);
-		increment_program_childcount(&threadinfo);
+		increment_program_childcount(&threadinfo, 0, 0);
 	}
 
 	sinsp_threadinfo& newentry = (m_threadtable[threadinfo.m_tid] = threadinfo);
@@ -743,13 +817,15 @@ void sinsp_thread_manager::add_thread(sinsp_threadinfo& threadinfo, bool from_sc
 	}
 }
 
-void sinsp_thread_manager::remove_thread(int64_t tid)
+void sinsp_thread_manager::remove_thread(int64_t tid, bool force)
 {
-	remove_thread(m_threadtable.find(tid));
+	remove_thread(m_threadtable.find(tid), force);
 }
 
-void sinsp_thread_manager::remove_thread(threadinfo_map_iterator_t it)
+void sinsp_thread_manager::remove_thread(threadinfo_map_iterator_t it, bool force)
 {
+	uint64_t nchilds;
+
 	if(it == m_threadtable.end())
 	{
 		//
@@ -763,7 +839,7 @@ void sinsp_thread_manager::remove_thread(threadinfo_map_iterator_t it)
 #endif
 		return;
 	}
-	else if(it->second.m_nchilds == 0)
+	else if((nchilds = it->second.m_nchilds) == 0 || force)
 	{
 		//
 		// Decrement the refcount of the main thread/program because
@@ -772,11 +848,17 @@ void sinsp_thread_manager::remove_thread(threadinfo_map_iterator_t it)
 		if(it->second.m_flags & PPM_CL_CLONE_THREAD)
 		{
 			ASSERT(it->second.m_pid != it->second.m_tid);
-			sinsp_threadinfo* main_thread = m_inspector->get_thread(it->second.m_pid, false);
+			sinsp_threadinfo* main_thread = m_inspector->get_thread(it->second.m_pid, false, true);
 			if(main_thread)
 			{
-				ASSERT(main_thread->m_nchilds);
-				--main_thread->m_nchilds;
+				if(main_thread->m_nchilds > 0)
+				{
+					--main_thread->m_nchilds;
+				}
+				else
+				{
+					ASSERT(false);
+				}
 			}
 			else
 			{
@@ -828,6 +910,17 @@ void sinsp_thread_manager::remove_thread(threadinfo_map_iterator_t it)
 #endif
 
 		m_threadtable.erase(it);
+
+		//
+		// If the thread has a nonzero refcount, it means that we are forcing the removal
+		// of a main process or program that some childs refer to.
+		// We need to recalculate the child relationships, or the table will become 
+		// corrupted.
+		//
+		if(nchilds != 0)
+		{
+			recreate_child_dependencies();
+		}
 	}
 }
 
@@ -843,11 +936,17 @@ void sinsp_thread_manager::remove_inactive_threads()
 	{
 		m_last_flush_time_ns = m_inspector->m_lastevent_ts;
 
+		g_logger.format(sinsp_logger::SEV_INFO, "Flusing thread table");
+
+		//
+		// Go through the table and remove dead entries.
+		//
 		for(threadinfo_map_iterator_t it = m_threadtable.begin(); it != m_threadtable.end();)
 		{
-			if(it->second.m_nchilds == 0 &&
-				m_inspector->m_lastevent_ts > 
-				it->second.m_lastaccess_ts + m_inspector->m_thread_timeout_ns)
+			bool closed = (it->second.m_flags & PPM_CL_CLOSED) != 0;
+
+			if(closed || 
+				(m_inspector->m_lastevent_ts > it->second.m_lastaccess_ts + m_inspector->m_thread_timeout_ns))
 			{
 				//
 				// Reset the cache
@@ -858,26 +957,68 @@ void sinsp_thread_manager::remove_inactive_threads()
 #ifdef GATHER_INTERNAL_STATS
 				m_removed_threads->increment();
 #endif
-				m_threadtable.erase(it++);
+				remove_thread(it++, closed);
 			}
 			else
 			{
 				++it;
 			}
 		}
+
+		//
+		// Rebalance the thread table dependency tree, so we free up threads that
+		// exited but that are stuck because of reference counting.
+		//
+		recreate_child_dependencies();
 	}
 }
 
 void sinsp_thread_manager::fix_sockets_coming_from_proc()
 {
 	threadinfo_map_iterator_t it;
-	for(it = m_threadtable.begin(); 
-		it != m_threadtable.end(); ++it)
+
+	for(it = m_threadtable.begin(); it != m_threadtable.end(); ++it)
 	{
 		it->second.fix_sockets_coming_from_proc();
 	}
 }
 
+void sinsp_thread_manager::reset_child_dependencies()
+{
+	threadinfo_map_iterator_t it;
+
+	m_last_tinfo = NULL;
+	m_last_tid = 0;
+
+	for(it = m_threadtable.begin(); it != m_threadtable.end(); ++it)
+	{
+		it->second.m_nchilds = 0;
+		it->second.m_main_program_thread = NULL;
+		it->second.m_progid = -1LL;
+		sinsp_fdtable* fdt = it->second.get_fd_table();
+		if(fdt != NULL)
+		{
+			fdt->m_last_accessed_fd = -1LL;
+		}
+	}
+}
+
+void sinsp_thread_manager::create_child_dependencies()
+{
+	threadinfo_map_iterator_t it;
+
+	for(it = m_threadtable.begin(); it != m_threadtable.end(); ++it)
+	{
+		increment_mainthread_childcount(&it->second);
+		increment_program_childcount(&it->second, 0, 0);
+	}
+}
+
+void sinsp_thread_manager::recreate_child_dependencies()
+{
+	reset_child_dependencies();
+	create_child_dependencies();
+}
 
 void sinsp_thread_manager::update_statistics()
 {
