@@ -21,6 +21,8 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <limits.h>
 #include <stdlib.h>
 #endif
+#include <algorithm> 
+#include <functional> 
 
 #include "sinsp.h"
 #include "sinsp_int.h"
@@ -28,7 +30,13 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "sinsp_signal.h"
 #include "filter.h"
 #include "filterchecks.h"
+#include "protodecoder.h"
 
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
+#ifdef HAS_CHISELS
 const chiseldir_info g_chisel_dirs_array[] =
 {
 	{false, ""}, // file as is
@@ -37,8 +45,9 @@ const chiseldir_info g_chisel_dirs_array[] =
 #endif
 	{false, "./"},
 	{false, "./chisels/"},
-	{true, "~/chisels/"},
+	{true, "~/.chisels/"},
 };
+#endif
 
 #ifndef _WIN32
 char* realpath_ex(const char *path, char *buff) 
@@ -70,7 +79,10 @@ sinsp_initializer g_initializer;
 #ifdef HAS_FILTERING
 sinsp_filter_check_list g_filterlist;
 #endif
+sinsp_protodecoder_list g_decoderlist;
+#ifdef HAS_CHISELS
 vector<chiseldir_info>* g_chisel_dirs = NULL;
+#endif
 
 //
 // loading time initializations
@@ -88,6 +100,7 @@ sinsp_initializer::sinsp_initializer()
 	//
 	g_logger.set_severity(sinsp_logger::SEV_DEBUG);
 
+#ifdef HAS_CHISELS
 	//
 	// Init the chisel directory list
 	//
@@ -124,6 +137,7 @@ sinsp_initializer::sinsp_initializer()
 			g_chisel_dirs->push_back(g_chisel_dirs_array[j]);
 		}
 	}
+#endif // HAS_CHISELS
 
 	//
 	// Sockets initialization on windows
@@ -137,10 +151,12 @@ sinsp_initializer::sinsp_initializer()
 
 sinsp_initializer::~sinsp_initializer()
 {
+#ifdef HAS_CHISELS
 	if(g_chisel_dirs)
 	{
 		delete g_chisel_dirs;
 	}
+#endif
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -267,6 +283,8 @@ const char* sinsp_utils::errno_to_str(int32_t code)
 		return "EPROTOTYPE";
 	case SE_EALREADY:
 		return "EALREADY";
+	case SE_ENOMEDIUM:
+		return "ENOMEDIUM";
 	default:
 		ASSERT(false);
 		return "";
@@ -642,6 +660,7 @@ bool sinsp_utils::concatenate_paths(char* target,
 	{
 		ASSERT(false);
 		strcpy(target, "/PATH_TOO_LONG");
+		return false;
 	}
 
 	if(len2 != 0 && path2[0] != '/')
@@ -685,7 +704,7 @@ const struct ppm_param_info* sinsp_utils::find_longest_matching_evt_param(string
 		{
 			const struct ppm_param_info* pi = &ei->params[k];
 			const char* an = pi->name;
-			uint32_t alen = strlen(an);
+			uint32_t alen = (uint32_t)strlen(an);
 			string subs = string(name, 0, alen);
 			
 			if(subs == an)
@@ -853,8 +872,11 @@ string ipv6serveraddr_to_string(ipv6serverinfo* addr)
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-// String split
+// String helpers
 ///////////////////////////////////////////////////////////////////////////////
+//
+// String split
+//
 vector<string> sinsp_split(const string &s, char delim)
 {
     vector<string> res;
@@ -869,6 +891,61 @@ vector<string> sinsp_split(const string &s, char delim)
 	return res;
 }
 
+//
+// trim from start
+//
+string& ltrim(string &s) 
+{
+	s.erase(s.begin(), find_if(s.begin(), s.end(), not1(ptr_fun<int, int>(isspace))));
+	return s;
+}
+
+//
+// trim from end
+//
+string& rtrim(string &s) 
+{
+	s.erase(find_if(s.rbegin(), s.rend(), not1(ptr_fun<int, int>(isspace))).base(), s.end());
+	return s;
+}
+
+//
+// trim from both ends
+//
+string& trim(string &s) 
+{
+	return ltrim(rtrim(s));
+}
+
+void replace_in_place(string &s, const string &search, const string &replace)
+{
+	for(size_t pos = 0; ; pos += replace.length()) 
+	{
+		// Locate the substring to replace
+		pos = s.find(search, pos);
+		if(pos == string::npos ) break;
+		// Replace by erasing and inserting
+		s.erase(pos, search.length());
+		s.insert(pos, replace );
+	}
+}
+
+void replace_in_place(string& str, string& substr_to_replace, string& new_substr) 
+{
+	size_t index = 0;
+	uint32_t nsize = (uint32_t)substr_to_replace.size();
+
+	while (true) 
+	{
+		 index = str.find(substr_to_replace, index);
+		 if (index == string::npos) break;
+
+		 str.replace(index, nsize, new_substr);
+
+		 index += nsize;
+	}
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // sinsp_numparser implementation
 ///////////////////////////////////////////////////////////////////////////////
@@ -1023,3 +1100,51 @@ bool sinsp_numparser::tryparsed64(const string& str, int64_t* res)
 
 	return true;
 }
+
+bool sinsp_numparser::tryparseu32_fast(const char* str, uint32_t strlen, uint32_t* res)
+{
+	const char* p = str;
+	const char* end = str + strlen;
+
+	*res = 0;
+
+	while(p < end)
+	{
+		if(*p >= '0' && *p <= '9')
+		{
+			*res = (*res) * 10 + (*p - '0');
+		}
+		else
+		{
+			return false;
+		}
+
+		p++;
+	}
+
+	return true;
+}
+
+bool sinsp_numparser::tryparsed32_fast(const char* str, uint32_t strlen, int32_t* res)
+{
+	const char* p = str;
+	const char* end = str + strlen;
+
+	*res = 0;
+
+	while(p < end)
+	{
+		if(*p >= '0' && *p <= '9')
+		{
+			*res = (*res) * 10 + (*p - '0');
+		}
+		else
+		{
+			return false;
+		}
+
+		p++;
+	}
+
+	return true;
+}
