@@ -38,7 +38,17 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <getopt.h>
 #endif
 
-bool ctrl_c_pressed = false;
+static bool g_terminate = false;
+#ifdef HAS_CHISELS
+vector<sinsp_chisel*> g_chisels;
+#endif
+
+
+
+// Sysdig 0.1.85 had log-rotation options (-C,-G,-W), but they were problematic,
+// so I'm disabling them until they can be fixed
+#define DISABLE_CGW
+
 
 static void usage();
 
@@ -47,15 +57,15 @@ static void usage();
 //
 static void signal_callback(int signal)
 {
-	ctrl_c_pressed = true;
+	g_terminate = true;
 }
 
-void replace_in_place(string& str, string substr_to_replace, string new_substr) 
+void replace_in_place(string& str, string substr_to_replace, string new_substr)
 {
 	size_t index = 0;
-	uint32_t nsize = substr_to_replace.size();
+	uint32_t nsize = (uint32_t)substr_to_replace.size();
 
-	while (true) 
+	while (true)
 	{
 		 index = str.find(substr_to_replace, index);
 		 if (index == string::npos) break;
@@ -75,36 +85,73 @@ static void usage()
 "sysdig version " SYSDIG_VERSION "\n"
 "Usage: sysdig [options] [-p <output_format>] [filter]\n\n"
 "Options:\n"
-" -A, --print-ascii  Only print the text portion of data buffers, and echo\n" 
+" -A, --print-ascii  Only print the text portion of data buffers, and echo\n"
 "                    end-of-lines. This is useful to only display human-readable\n"
 "                    data.\n"
-" -a, --abstime      Show absolute event timestamps\n"
+#ifdef HAS_CHISELS
 " -c <chiselname> <chiselargs>, --chisel  <chiselname> <chiselargs>\n"
 "                    run the specified chisel. If the chisel require arguments,\n"
 "                    they must be specified in the command line after the name.\n"
 " -cl, --list-chisels\n"
-"                    lists the available chisels. Looks for chisels in .,\n" 
-"                    ./chisels, ~/chisels and /usr/share/sysdig/chisels.\n"
+"                    lists the available chisels. Looks for chisels in .,\n"
+"                    ./chisels, ~/.chisels and /usr/share/sysdig/chisels.\n"
+#endif
+#ifndef DISABLE_CGW
+" -C <file_size>, --file-size=<file_size>\n"
+"                    Before writing an event, check whether the file is\n"
+"                    currently larger than file_size and, if so, close the\n"
+"                    current file and open a new one. Savefiles will have the\n"
+"                    name specified with the -w flag, with a number after it,\n"
+"                    starting at 0 and continuing upward. The units of file_size\n"
+"                    are millions of bytes (10^6, not 2^20). Use the -W flag to\n"
+"                    determine how many files will be saved to disk.\n"
+#endif
 " -d, --displayflt   Make the given filter a display one\n"
-"                    Setting this option causes the events to be filtered\n" 
+"                    Setting this option causes the events to be filtered\n"
 "                    after being parsed by the state system. Events are\n"
 "                    normally filtered before being analyzed, which is more\n"
-"                    efficient, but can cause state (e.g. FD names) to be lost\n"
+"                    efficient, but can cause state (e.g. FD names) to be lost.\n"
+" -D, --debug        Capture events about sysdig itself\n"
+" -F, --fatfile	     Enable fatfile mode\n"
+"                    when writing in fatfile mode, the output file will contain\n"
+"                    events that will be invisible when reading the file, but\n"
+"                    that are necessary to fully reconstruct the state.\n"
+"                    Fatfile mode is useful when saving events to disk with an\n"
+"                    aggressive filter. The filter could drop events that would\n"
+"                    the state to be updated (e.g. clone() or open()). With\n"
+"                    fatfile mode, those events are still saved to file, but\n"
+"                    'hidden' so that they won't appear when reading the file.\n"
+"                    Be aware that using this flag might generate substantially\n"
+"                    bigger traces files.\n"
+#ifndef DISABLE_CGW
+" -G <num_seconds>, --seconds=<num_seconds>\n"
+"                    Rotates the dump file specified with the -w option every\n"
+"                    num_seconds seconds. Savefiles will have the name specified\n"
+"                    by -w which should include a time format as defined by strftime(3).\n"
+"                    If no time format is specified, each new file will overwrite the\n"
+"                    previous.\n"
+"\n"
+"                    If used in conjunction with the -C option, filenames will take\n"
+"                    the form of `file<count>'.\n"
+#endif
 " -h, --help         Print this page\n"
+#ifdef HAS_CHISELS
 " -i <chiselname>, --chisel-info <chiselname>\n"
 "                    Get a longer description and the arguments associated with\n"
 "                    a chisel found in the -cl option list.\n"
+#endif
 " -j, --json         Emit output as json\n"
+" -L, --list-events  List the events that the engine supports\n"
 " -l, --list         List the fields that can be used for filtering and output\n"
 "                    formatting. Use -lv to get additional information for each\n"
 "                    field.\n"
-" -L, --list-events  List the events that the engine supports\n"
 " -n <num>, --numevents=<num>\n"
 "                    Stop capturing after <num> events\n"
+" -P, --progress     Print progress on stderr while processing trace files\n"
 " -p <output_format>, --print=<output_format>\n"
 "                    Specify the format to be used when printing the events.\n"
 "                    See the examples section below for more info.\n"
-" -q, --quiet        Don't print events on the screen.\n"
+" -q, --quiet        Don't print events on the screen\n"
 "                    Useful when dumping to disk.\n"
 " -r <readfile>, --read=<readfile>\n"
 "                    Read the events from <readfile>.\n"
@@ -116,27 +163,42 @@ static void usage()
 "                    option with caution, it can generate huge trace files.\n"
 " -t <timetype>, --timetype=<timetype>\n"
 "                    Change the way event time is diplayed. Accepted values are\n"
-"                    h for human-readable string, a for abosulte timestamp from\n" 
-"                    epoch, r for relative time from the beginning of the\n" 
-"                    capture, and d for delta between event enter and exit.\n" 
+"                    h for human-readable string, a for absolute timestamp from\n"
+"                    epoch, r for relative time from the beginning of the\n"
+"                    capture, and d for delta between event enter and exit.\n"
 " -v, --verbose      Verbose output.\n"
 " -w <writefile>, --write=<writefile>\n"
 "                    Write the captured events to <writefile>.\n"
-" -x, --print-hex   Print data buffers in hex.\n"
+#ifndef DISABLE_CGW
+" -W <num>, --limit <num>\n"
+"                    Used in conjunction with the -C option, this will limit the number\n"
+"                    of files created to the specified number, and begin overwriting files\n"
+"                    from the beginning, thus creating a 'rotating' buffer. In addition, it\n"
+"                    will name the files with enough leading 0s to support the maximum number\n"
+"                    of files, allowing them to sort correctly.\n"
+"\n"
+"                    Used in conjunction with the -G option, this will limit the number\n"
+"                    of rotated dump files that get created, exiting with status 0 when\n"
+"                    reaching the limit. If used with -C as well, the behavior will result\n"
+"                    in cyclical files per timeslice.\n"
+#endif
+" -x, --print-hex    Print data buffers in hex.\n"
 " -X, --print-hex-ascii\n"
 "                    Print data buffers in hex and ASCII.\n"
+" -z, --compress     Used with -w, enables compression for tracefiles.\n"
 "\n"
 "Output format:\n\n"
 "By default, sysdig prints the information for each captured event on a single\n"
 " line with the following format:\n\n"
-"<evt.time> %%evt.cpu %%proc.name (%%thread.tid) %%evt.dir %%evt.type %%evt.args\n\n"
+" %%evt.num %%evt.time %%evt.cpu %%proc.name (%%thread.tid) %%evt.dir %%evt.type %%evt.args\n\n"
 "where:\n"
+" evt.num is the incremental event number\n"
 " evt.time is the event timestamp\n"
 " evt.cpu is the CPU number where the event was captured\n"
 " proc.name is the name of the process that generated the event\n"
-" thread.tid id the TID that generated the event, which corresponds to the\n" 
-"   PID for single thread processes\n" 
-" evt.dir is the event direction, > for enter events and < for exit events\n" 
+" thread.tid id the TID that generated the event, which corresponds to the\n"
+"   PID for single thread processes\n"
+" evt.dir is the event direction, > for enter events and < for exit events\n"
 " evt.type is the name of the event, e.g. 'open' or 'read'\n"
 " evt.args is the list of event arguments.\n\n"
 "The output format can be customized with the -p switch, using any of the\n"
@@ -151,12 +213,12 @@ static void usage()
 " Print all the open system calls invoked by cat\n"
 "   $ sysdig proc.name=cat and evt.type=open\n\n"
 " Print the name of the files opened by cat\n"
-"   $ ./sysdig -p\"%%evt.arg.name\" proc.name=cat and evt.type=open\n\n"
+"   $ sysdig -p\"%%evt.arg.name\" proc.name=cat and evt.type=open\n\n"
     );
 }
 
-void print_summary_table(sinsp* inspector, 
-						 vector<summary_table_entry>* summary_table, 
+void print_summary_table(sinsp* inspector,
+						 vector<summary_table_entry>* summary_table,
 						 uint32_t nentries)
 {
 	sinsp_evttables* einfo = inspector->get_event_info_tables();
@@ -168,7 +230,7 @@ void print_summary_table(sinsp* inspector,
 	cout << tstr;
 	cout << "----------------------\n";
 
-	sort(summary_table->begin(), summary_table->end(), 
+	sort(summary_table->begin(), summary_table->end(),
 		summary_table_entry_rsort_comparer());
 
 	for(uint32_t j = 0; j < nentries; j++)
@@ -185,9 +247,9 @@ void print_summary_table(sinsp* inspector,
 			tstr = einfo->m_syscall_info_table[e->m_id / 2].name;
 			tstr.resize(16, ' ');
 
-			printf("%s%s%" PRIu64 "\n", 
+			printf("%s%s%" PRIu64 "\n",
 				(PPME_IS_ENTER(e->m_id))? "> ": "< ",
-				tstr.c_str(), 
+				tstr.c_str(),
 				e->m_ncalls);
 		}
 		else
@@ -195,23 +257,198 @@ void print_summary_table(sinsp* inspector,
 			tstr = einfo->m_event_info[e->m_id].name;
 			tstr.resize(16, ' ');
 
-			printf("%s%s%" PRIu64 "\n", 
+			printf("%s%s%" PRIu64 "\n",
 				(PPME_IS_ENTER(e->m_id))? "> ": "< ",
-				tstr.c_str(), 
+				tstr.c_str(),
 				e->m_ncalls);
 		}
 	}
 }
 
+#ifdef HAS_CHISELS
+static void add_chisel_dirs(sinsp* inspector)
+{
+	//
+	// Add the default chisel directory statically configured by the build system
+	//
+	inspector->add_chisel_dir(SYSDIG_INSTALLATION_DIR CHISELS_INSTALLATION_DIR, false);
+
+	//
+	// Add the directories configured in the SYSDIG_CHISEL_DIR environment variable
+	//
+	char* s_user_cdirs = getenv("SYSDIG_CHISEL_DIR");
+
+	if(s_user_cdirs != NULL)
+	{
+		vector<string> user_cdirs = sinsp_split(s_user_cdirs, ';');
+
+		for(uint32_t j = 0; j < user_cdirs.size(); j++)
+		{
+			inspector->add_chisel_dir(user_cdirs[j], true);
+		}
+	}
+}
+#endif
+
+static void initialize_chisels()
+{
+#ifdef HAS_CHISELS
+	for(uint32_t j = 0; j < g_chisels.size(); j++)
+	{
+		g_chisels[j]->on_init();
+	}
+#endif
+}
+
+//
+// Parse the command line following a chisel to consume the chisel command line.
+// We use the following strategy:
+//  - if the chisel has no arguments, we don't consume anything
+//  - if the chisel has at least one required argument, we consume the next command line token
+//  - if the chisel has only optional arguments, we consume the next token, unless
+//    - there is no next token
+//    - the next token starts with a '-'
+//    - the rest of the command line contains a valid filter
+//
+static void parse_chisel_args(sinsp_chisel* ch, sinsp* inspector, int optind, int argc, char **argv, int32_t* n_filterargs)
+{
+	uint32_t nargs = ch->get_n_args();
+	uint32_t nreqargs = ch->get_n_required_args();
+	string args;
+
+	if(nargs != 0)
+	{
+		if(optind > (int32_t)argc)
+		{
+			throw sinsp_exception("invalid number of arguments for chisel " + string(optarg) + ", " + to_string((long long int)nargs) + " expected.");
+		}
+		else if(optind < (int32_t)argc)
+		{
+			args = argv[optind];
+
+			if(nreqargs != 0)
+			{
+				ch->set_args(args);
+				(*n_filterargs)++;
+			}
+			else
+			{
+				if(args[0] != '-')
+				{
+					string testflt;
+
+					for(int32_t j = optind; j < argc; j++)
+					{
+						testflt += argv[j];
+						if(j < argc - 1)
+						{
+							testflt += " ";
+						}
+					}
+
+					try
+					{
+						sinsp_filter df(inspector, testflt);
+					}
+					catch(...)
+					{
+						ch->set_args(args);
+						(*n_filterargs)++;
+					}
+				}
+			}
+		}
+		else
+		{
+			if(nreqargs != 0)
+			{
+				throw sinsp_exception("missing arguments for chisel " + string(optarg));
+			}
+		}
+	}
+}
+
+static void free_chisels()
+{
+#ifdef HAS_CHISELS
+	for(vector<sinsp_chisel*>::iterator it = g_chisels.begin();
+		it != g_chisels.end(); ++it)
+	{
+		delete *it;
+	}
+
+	g_chisels.clear();
+#endif
+}
+
+static void chisels_on_capture_start()
+{
+#ifdef HAS_CHISELS
+	for(uint32_t j = 0; j < g_chisels.size(); j++)
+	{
+		g_chisels[j]->on_capture_start();
+	}
+#endif
+}
+
+static void chisels_on_capture_end()
+{
+#ifdef HAS_CHISELS
+	for(vector<sinsp_chisel*>::iterator it = g_chisels.begin();
+		it != g_chisels.end(); ++it)
+	{
+		(*it)->on_capture_end();
+	}
+#endif
+}
+
+static void chisels_do_timeout(sinsp_evt* ev)
+{
+#ifdef HAS_CHISELS
+	for(vector<sinsp_chisel*>::iterator it = g_chisels.begin();
+		it != g_chisels.end(); ++it)
+	{
+		(*it)->do_timeout(ev);
+	}
+#endif
+}
+
+void handle_end_of_file(bool print_progress, sinsp_evt_formatter* formatter = NULL)
+{
+	string line;
+
+	// Notify the formatter that we are at the
+	// end of the capture in case it needs to
+	// write any terminating characters
+	if(formatter != NULL && formatter->on_capture_end(&line))
+	{
+		cout << line << endl;
+	}
+
+	//
+	// Reached the end of a trace file.
+	// If we are reporting prgress, this is 100%
+	//
+	if(print_progress)
+	{
+		fprintf(stderr, "100.00\n");
+		fflush(stderr);
+	}
+
+	//
+	// Notify the chisels that we're exiting.
+	//
+	chisels_on_capture_end();
+}
+
 //
 // Event processing loop
 //
-captureinfo do_inspect(sinsp* inspector, 
-					   uint64_t cnt, 
-					   bool quiet, 
-					   bool absolute_times,
+captureinfo do_inspect(sinsp* inspector,
+					   uint64_t cnt,
+					   bool quiet,
+					   bool print_progress,
 					   sinsp_filter* display_filter,
-					   vector<sinsp_chisel*>* chisels,
 					   vector<summary_table_entry>* summary_table,
 					   sinsp_evt_formatter* formatter)
 {
@@ -222,25 +459,20 @@ captureinfo do_inspect(sinsp* inspector,
 	uint64_t deltats = 0;
 	uint64_t firstts = 0;
 	string line;
+	double last_printed_progress_pct = 0;
 
 	//
 	// Loop through the events
 	//
 	while(1)
 	{
-		if(retval.m_nevts == cnt || ctrl_c_pressed)
+		if(retval.m_nevts == cnt || g_terminate)
 		{
 			//
 			// End of capture, either because the user stopped it, or because
 			// we reached the event count specified with -n.
-			// Notify the chisels that we're exiting.
 			//
-			for(vector<sinsp_chisel*>::iterator it = chisels->begin(); 
-				it != chisels->end(); ++it)
-			{
-				(*it)->on_capture_end();
-			}
-
+			handle_end_of_file(print_progress, formatter);
 			break;
 		}
 
@@ -254,24 +486,14 @@ captureinfo do_inspect(sinsp* inspector,
 				// The event has been dropped by the filtering system.
 				// Give the chisels a chance to run their timeout logic.
 				//
-				for(vector<sinsp_chisel*>::iterator it = chisels->begin(); it != chisels->end(); ++it)
-				{
-					(*it)->do_timeout(ev);
-				}
+				chisels_do_timeout(ev);
 			}
+
 			continue;
 		}
 		else if(res == SCAP_EOF)
 		{
-			//
-			// Reached the end of a trace file.
-			// Notify the chisels that we're exiting.
-			//
-			for(vector<sinsp_chisel*>::iterator it = chisels->begin(); it != chisels->end(); ++it)
-			{
-				(*it)->on_capture_end();
-			}
-
+			handle_end_of_file(print_progress, formatter);
 			break;
 		}
 		else if(res != SCAP_SUCCESS)
@@ -280,11 +502,7 @@ captureinfo do_inspect(sinsp* inspector,
 			// Event read error.
 			// Notify the chisels that we're exiting, and then die with an error.
 			//
-			for(vector<sinsp_chisel*>::iterator it = chisels->begin(); it != chisels->end(); ++it)
-			{
-				(*it)->on_capture_end();
-			}
-
+			handle_end_of_file(print_progress, formatter);
 			cerr << "res = " << res << endl;
 			throw sinsp_exception(inspector->getlasterr().c_str());
 		}
@@ -298,12 +516,28 @@ captureinfo do_inspect(sinsp* inspector,
 		}
 		deltats = ts - firstts;
 
+		if(print_progress)
+		{
+			if(ev->get_num() % 10000 == 0)
+			{
+				double progress_pct = inspector->get_read_progress();
+
+				if(progress_pct - last_printed_progress_pct > 0.1)
+				{
+					fprintf(stderr, "%.2lf\n", progress_pct);
+					fflush(stderr);
+					last_printed_progress_pct = progress_pct;
+				}
+			}
+		}
+
 		//
 		// If there are chisels to run, run them
 		//
-		if(!chisels->empty())
+#ifdef HAS_CHISELS
+		if(!g_chisels.empty())
 		{
-			for(vector<sinsp_chisel*>::iterator it = chisels->begin(); it != chisels->end(); ++it)
+			for(vector<sinsp_chisel*>::iterator it = g_chisels.begin(); it != g_chisels.end(); ++it)
 			{
 				if((*it)->run(ev) == false)
 				{
@@ -312,14 +546,15 @@ captureinfo do_inspect(sinsp* inspector,
 			}
 		}
 		else
-		{		
+#endif
+		{
 			//
 			// If we're supposed to summarize, increase the count for this event
 			//
 			if(summary_table != NULL)
 			{
 				uint16_t etype = ev->get_type();
-				
+
 				if(etype == PPME_GENERIC_E)
 				{
 					sinsp_evt_param *parinfo = ev->get_param(0);
@@ -347,20 +582,28 @@ captureinfo do_inspect(sinsp* inspector,
 				continue;
 			}
 
-			//
-			// Output the line
-			//
-			if(display_filter)
-			{
-				if(!display_filter->run(ev))
-				{
-					continue;
-				}
-			}
-
 			if(formatter->tostring(ev, &line))
 			{
-				cout << line << endl;
+				//
+				// Output the line
+				//
+				if(display_filter)
+				{
+					if(!display_filter->run(ev))
+					{
+						continue;
+					}
+				}
+
+				cout << line;
+				if( inspector->get_buffer_format() != sinsp_evt::PF_JSON)
+				{
+					cout << endl;
+				}
+				else
+				{
+					cout << flush;
+				}
 			}
 		}
 	}
@@ -370,22 +613,22 @@ captureinfo do_inspect(sinsp* inspector,
 }
 
 //
-// MAIN
+// ARGUMENT PARSING AND PROGRAM SETUP
 //
-int main(int argc, char **argv)
+sysdig_init_res sysdig_init(int argc, char **argv)
 {
-	int res = EXIT_SUCCESS;
+	sysdig_init_res res;
 	sinsp* inspector = NULL;
-	string infile;
+	vector<string> infiles;
 	string outfile;
 	int op;
 	uint64_t cnt = -1;
-	bool emitjson = false;
 	bool quiet = false;
-	bool absolute_times = false;
 	bool is_filter_display = false;
 	bool verbose = false;
 	bool list_flds = false;
+	bool print_progress = false;
+	bool compress = false;
 	sinsp_evt::param_fmt event_buffer_format = sinsp_evt::PF_NORMAL;
 	sinsp_filter* display_filter = NULL;
 	double duration = 1;
@@ -393,26 +636,44 @@ int main(int argc, char **argv)
 	string output_format;
 	uint32_t snaplen = 0;
 	int long_index = 0;
-	vector<sinsp_chisel*> chisels;
 	int32_t n_filterargs = 0;
 	int cflag = 0;
 	string cname;
 	vector<summary_table_entry>* summary_table = NULL;
 	string timefmt = "%evt.time";
 
-	static struct option long_options[] = 
+	// These variables are for the cycle_writer engine
+	int duration_seconds = 0;	
+	int rollover_mb = 0;
+	int file_limit = 0;
+	bool do_cycle = false;
+
+	static struct option long_options[] =
 	{
 		{"print-ascii", no_argument, 0, 'A' },
-		{"abstimes", no_argument, 0, 'a' },
+#ifdef HAS_CHISELS
 		{"chisel", required_argument, 0, 'c' },
 		{"list-chisels", no_argument, &cflag, 1 },
+#endif
+		{"compress", no_argument, 0, 'z' },
 		{"displayflt", no_argument, 0, 'd' },
+		{"debug", no_argument, 0, 'D'},
+		{"fatfile", no_argument, 0, 'F'},
+#ifndef DISABLE_CGW
+		{"seconds", required_argument, 0, 'G' },
+#endif
 		{"help", no_argument, 0, 'h' },
+#ifdef HAS_CHISELS
 		{"chisel-info", required_argument, 0, 'i' },
+#endif
+#ifndef DISABLE_CGW
+		{"file-size", required_argument, 0, 'C' },
+#endif
 		{"json", no_argument, 0, 'j' },
 		{"list", no_argument, 0, 'l' },
 		{"list-events", no_argument, 0, 'L' },
 		{"numevents", required_argument, 0, 'n' },
+		{"progress", required_argument, 0, 'P' },
 		{"print", required_argument, 0, 'p' },
 		{"quiet", no_argument, 0, 'q' },
 		{"readfile", required_argument, 0, 'r' },
@@ -421,24 +682,42 @@ int main(int argc, char **argv)
 		{"timetype", required_argument, 0, 't' },
 		{"verbose", no_argument, 0, 'v' },
 		{"writefile", required_argument, 0, 'w' },
+#ifndef DISABLE_CGW
+		{"limit", required_argument, 0, 'W' },
+#endif
 		{"print-hex", no_argument, 0, 'x'},
 		{"print-hex-ascii", no_argument, 0, 'X'},
 		{0, 0, 0, 0}
 	};
 
-	output_format = "*%evt.num <TIME> %evt.cpu %proc.name (%thread.tid) %evt.dir %evt.type %evt.args";
+	output_format = "*%evt.num <TIME> %evt.cpu %proc.name (%thread.tid) %evt.dir %evt.type %evt.info";
 //	output_format = DEFAULT_OUTPUT_STR;
 
 	try
 	{
 		inspector = new sinsp();
 
-		inspector->add_chisel_dir(SYSDIG_INSTALLATION_DIR CHISELS_INSTALLATION_DIR);
+#ifdef HAS_CHISELS
+		add_chisel_dirs(inspector);
+#endif
 
 		//
 		// Parse the args
 		//
-		while((op = getopt_long(argc, argv, "Aac:dhi:jlLn:p:qr:Ss:t:vw:xX", long_options, &long_index)) != -1)
+		while((op = getopt_long(argc, argv,
+                                        "Ac:"
+#ifndef DISABLE_CGW
+                                        "C:"
+#endif
+                                        "dDF"
+#ifndef DISABLE_CGW
+                                        "G:"
+#endif
+                                        "hi:jlLn:Pp:qr:Ss:t:v"
+#ifndef DISABLE_CGW
+                                        "W:"
+#endif
+                                        "w:xXz", long_options, &long_index)) != -1)
 		{
 			switch(op)
 			{
@@ -447,14 +726,11 @@ int main(int argc, char **argv)
 				{
 					fprintf(stderr, "you cannot specify more than one output format\n");
 					delete inspector;
-					return EXIT_SUCCESS;
+					return sysdig_init_res(EXIT_SUCCESS);
 				}
 
 				event_buffer_format = sinsp_evt::PF_EOLS;
 				break;
-			case 'a':
-				absolute_times = true;
-				break;
 			case 0:
 				if(cflag != 1 && cflag != 2)
 				{
@@ -465,6 +741,7 @@ int main(int argc, char **argv)
 				{
 					cname = optarg;
 				}
+#ifdef HAS_CHISELS
 			case 'c':
 				{
 					if(cflag == 0)
@@ -484,32 +761,54 @@ int main(int argc, char **argv)
 					{
 						vector<chisel_desc> chlist;
 						sinsp_chisel::get_chisel_list(&chlist);
-						list_chisels(&chlist);
+						list_chisels(&chlist, true);
 						delete inspector;
-						return EXIT_SUCCESS;
+						return sysdig_init_res(EXIT_SUCCESS);
 					}
 
 					sinsp_chisel* ch = new sinsp_chisel(inspector, optarg);
-					uint32_t nargs = ch->get_n_args();
-					vector<string> args;
-
-					for(uint32_t j = 0; j < nargs; j++)
-					{
-						if(optind + j >= (uint32_t)argc)
-						{
-							throw sinsp_exception("invalid number of arguments for chisel " + string(optarg) + ", " + to_string((long long int)nargs) + " expected.");
-						}
-
-						args.push_back(argv[optind + j]);
-						n_filterargs++;
-					}
-
-					ch->set_args(&args);
-
-					chisels.push_back(ch);
+					parse_chisel_args(ch, inspector, optind, argc, argv, &n_filterargs);
+					g_chisels.push_back(ch);
 				}
+#endif
 				break;
 
+#ifndef DISABLE_CGW
+			// File-size
+			case 'C':
+				rollover_mb = atoi(optarg);
+				if(rollover_mb <= 0)
+				{
+					throw sinsp_exception(string("invalid file size") + optarg);
+					res.m_res = EXIT_FAILURE;
+					goto exit;
+				}
+
+				// -C always implicates a cycle
+				do_cycle = true;
+				break;
+#endif
+
+			case 'D':
+				inspector->set_debug_mode(true);
+				break;
+			case 'F':
+				inspector->set_fatfile_dump_mode(true);
+				break;
+#ifndef DISABLE_CGW
+			// Number of seconds between roll-over
+			case 'G':
+				duration_seconds = atoi(optarg);
+				if(duration_seconds <= 0)
+				{
+					throw sinsp_exception(string("invalid duration") + optarg);
+					res.m_res = EXIT_FAILURE;
+					goto exit;
+				}
+				break;
+#endif
+
+#ifdef HAS_CHISELS
 			// --chisel-info and -i
 			case 'i':
 				{
@@ -525,47 +824,51 @@ int main(int argc, char **argv)
 						{
 							print_chisel_info(&chlist[j]);
 							delete inspector;
-							return EXIT_SUCCESS;
+							return sysdig_init_res(EXIT_SUCCESS);
 						}
 					}
 
 					throw sinsp_exception("chisel " + cname + " not found - use -cl to list them.");
-				}	
+				}
 				break;
+#endif
 
 			case 'd':
 				is_filter_display = true;
 				break;
 			case 'j':
+				if(event_buffer_format != sinsp_evt::PF_NORMAL)
 				{
-					emitjson = true;
-					if (emitjson)
-					{
-						ASSERT(false);
-						throw sinsp_exception("json option not yet implemented");
-					}
+					fprintf(stderr, "you cannot specify more than one output format\n");
+					delete inspector;
+					return sysdig_init_res(EXIT_SUCCESS);
 				}
+
+				event_buffer_format = sinsp_evt::PF_JSON;
 				break;
 			case 'h':
 				usage();
 				delete inspector;
-				return EXIT_SUCCESS;
+				return sysdig_init_res(EXIT_SUCCESS);
 			case 'l':
 				list_flds = true;
 				break;
 			case 'L':
 				list_events(inspector);
 				delete inspector;
-				return EXIT_SUCCESS;
+				return sysdig_init_res(EXIT_SUCCESS);
 			case 'n':
 				cnt = atoi(optarg);
 				if(cnt <= 0)
 				{
-					throw sinsp_exception(string("invalid packet count") + optarg);
-					res = EXIT_FAILURE;
+					throw sinsp_exception(string("invalid event count ") + optarg);
+					res.m_res = EXIT_FAILURE;
 					goto exit;
 				}
 				break;
+			case 'P':
+				print_progress = true;
+				break;
 			case 'p':
 				if(string(optarg) == "p")
 				{
@@ -575,7 +878,7 @@ int main(int argc, char **argv)
 					replace_in_place(output_format, "<TIME>", timefmt);
 					printf("%s\n", output_format.c_str());
 					delete inspector;
-					return EXIT_SUCCESS;
+					return sysdig_init_res(EXIT_SUCCESS);
 				}
 				else
 				{
@@ -587,14 +890,14 @@ int main(int argc, char **argv)
 				quiet = true;
 				break;
 			case 'r':
-				infile = optarg;
+				infiles.push_back(optarg);
 				break;
 			case 'S':
 				summary_table = new vector<summary_table_entry>;
 
 				for(uint32_t j = 0; j < PPM_EVENT_MAX; j++)
 				{
-					summary_table->push_back(summary_table_entry(j, false)); 
+					summary_table->push_back(summary_table_entry(j, false));
 				}
 
 				for(uint32_t j = 0; j < PPM_SC_MAX * 2; j++)
@@ -635,12 +938,26 @@ int main(int argc, char **argv)
 				outfile = optarg;
 				quiet = true;
 				break;
+
+#ifndef DISABLE_CGW
+			// Number of capture files to cycle through
+			case 'W':
+				file_limit = atoi(optarg);
+				if(file_limit <= 0)
+				{
+					throw sinsp_exception(string("invalid file limit") + optarg);
+					res.m_res = EXIT_FAILURE;
+					goto exit;
+				}
+				break;
+#endif
+
 			case 'x':
 				if(event_buffer_format != sinsp_evt::PF_NORMAL)
 				{
 					fprintf(stderr, "you cannot specify more than one output format\n");
 					delete inspector;
-					return EXIT_SUCCESS;
+					return sysdig_init_res(EXIT_SUCCESS);
 				}
 
 				event_buffer_format = sinsp_evt::PF_HEX;
@@ -650,11 +967,14 @@ int main(int argc, char **argv)
 				{
 					fprintf(stderr, "you cannot specify more than one output format\n");
 					delete inspector;
-					return EXIT_SUCCESS;
+					return sysdig_init_res(EXIT_SUCCESS);
 				}
 
 				event_buffer_format = sinsp_evt::PF_HEXASCII;
 				break;
+			case 'z':
+				compress = true;
+				break;
 			default:
 				break;
 			}
@@ -680,18 +1000,18 @@ int main(int argc, char **argv)
 				list_fields(false);
 			}
 
-			res = EXIT_SUCCESS;
+			res.m_res = EXIT_SUCCESS;
 			goto exit;
 		}
 
+		string filter;
+
 		//
 		// the filter is at the end of the command line
 		//
 		if(optind + n_filterargs < argc)
 		{
-	#ifdef HAS_FILTERING
-			string filter;
-
+#ifdef HAS_FILTERING
 			for(int32_t j = optind + n_filterargs; j < argc; j++)
 			{
 				filter += argv[j];
@@ -701,175 +1021,189 @@ int main(int argc, char **argv)
 				}
 			}
 
-			try
+			if(is_filter_display)
 			{
-				if(is_filter_display)
-				{
-					display_filter = new sinsp_filter(inspector, filter);
-				}
-				else
-				{
-					inspector->set_filter(filter);
-				}
+				display_filter = new sinsp_filter(inspector, filter);
 			}
-			catch(sinsp_exception e)
-			{
-				cerr << e.what() << endl;
-				res = EXIT_FAILURE;
-				goto exit;
-			}
-	#else
+#else
 			fprintf(stderr, "filtering not compiled.\n");
-			res = EXIT_FAILURE;
+			res.m_res = EXIT_FAILURE;
 			goto exit;
-	#endif
+#endif
 		}
 
-		//
-		// Set the CRTL+C signal
-		//
 		if(signal(SIGINT, signal_callback) == SIG_ERR)
 		{
-			fprintf(stderr, "An error occurred while setting a signal handler.\n");
-			res = EXIT_FAILURE;
+			fprintf(stderr, "An error occurred while setting SIGINT signal handler.\n");
+			res.m_res = EXIT_FAILURE;
+			goto exit;
+		}
+
+		if(signal(SIGTERM, signal_callback) == SIG_ERR)
+		{
+			fprintf(stderr, "An error occurred while setting SIGTERM signal handler.\n");
+			res.m_res = EXIT_FAILURE;
 			goto exit;
 		}
 
 		//
 		// Insert the right time format based on the -t flag
 		//
-		replace_in_place(output_format, "<TIME>", timefmt); 
+		replace_in_place(output_format, "<TIME>", timefmt);
 
 		//
 		// Create the event formatter
 		//
 		sinsp_evt_formatter formatter(inspector, output_format);
 
-		//
-		// Initialize the chisels
-		//
-		for(uint32_t j = 0; j < chisels.size(); j++)
+		for(uint32_t j = 0; j < infiles.size() || infiles.size() == 0; j++)
 		{
-			chisels[j]->on_init();
-		}
-
-		//
-		// Launch the capture
-		//
-		bool open_success = true;
-
-		if(infile != "")
-		{
-			//
-			// We have a file to open
-			//
-			inspector->open(infile);				
-		}
-		else
-		{
-			//
-			// No file to open, this is a live capture
-			//
-			try
+#ifdef HAS_FILTERING
+			if(filter.size() && !is_filter_display)
 			{
-				inspector->open("");
-			}
-			catch(sinsp_exception e)
-			{
-				open_success = false;
+				inspector->set_filter(filter);
 			}
+#endif
 
 			//
-			// Starting the live capture failed, try to load the driver with
-			// modprobe.
+			// Launch the capture
 			//
-			if(!open_success)
+			bool open_success = true;
+
+			if(infiles.size() != 0)
 			{
-				open_success = true;
+				initialize_chisels();
+
+				//
+				// We have a file to open
+				//
+				inspector->open(infiles[j]);
+			}
+			else
+			{
+				if(j > 0)
+				{
+					break;
+				}
+
+				initialize_chisels();
+
+				//
+				// No file to open, this is a live capture
+				//
+#if defined(HAS_CAPTURE)
+				if(print_progress)
+				{
+					fprintf(stderr, "the -P flag cannot be used with live captures.\n");
+					res.m_res = EXIT_FAILURE;
+					goto exit;
+				}
 
 				try
 				{
-					system("modprobe sysdig-probe > /dev/null 2> /dev/null");
-
 					inspector->open("");
 				}
 				catch(sinsp_exception e)
 				{
 					open_success = false;
-				}			
-			}
-
-			//
-			// No luck with modprobe either.
-			// Maybe this is a version of sysdig that was compiled from the 
-			// sources, so let's make one last attempt with insmod and the
-			// path to the driver directory.
-			//
-			if(!open_success)
-			{
-				system("insmod ../../driver/sysdig-probe.ko > /dev/null 2> /dev/null");
-				
+				}
+#else
+				//
+				// Starting live capture
+				// If this fails on Windows and OSX, don't try with any driver
+				//
 				inspector->open("");
+#endif
+
+				//
+				// Starting the live capture failed, try to load the driver with
+				// modprobe.
+				//
+				if(!open_success)
+				{
+					open_success = true;
+
+					system("modprobe sysdig-probe > /dev/null 2> /dev/null");
+
+					inspector->open("");
+				}
 			}
-		}
 
-		if(snaplen != 0)
-		{
-			inspector->set_snaplen(snaplen);
-		}
+			if(snaplen != 0)
+			{
+				inspector->set_snaplen(snaplen);
+			}
 
-		if(outfile != "")
-		{
-			inspector->autodump_start(outfile);
-		}
+			duration = ((double)clock()) / CLOCKS_PER_SEC;
 
-		duration = ((double)clock()) / CLOCKS_PER_SEC;
-			
-		//
-		// Notify the chisels that the capture is starting
-		//
-		for(uint32_t j = 0; j < chisels.size(); j++)
-		{
-			chisels[j]->on_capture_start();
-		}
+			if(outfile != "")
+			{
+				inspector->setup_cycle_writer(outfile, rollover_mb, duration_seconds, file_limit, do_cycle, compress);
+				inspector->autodump_next_file();
+			}
 
-		cinfo = do_inspect(inspector, 
-			cnt, 
-			quiet, 
-			absolute_times,
-			display_filter,
-			&chisels,
-			summary_table,
-			&formatter);
+			//
+			// Notify the chisels that the capture is starting
+			//
+			chisels_on_capture_start();
 
-		duration = ((double)clock()) / CLOCKS_PER_SEC - duration;
+			cinfo = do_inspect(inspector,
+				cnt,
+				quiet,
+				print_progress,
+				display_filter,
+				summary_table,
+				&formatter);
 
-		scap_stats cstats;
-		inspector->get_capture_stats(&cstats);
+			duration = ((double)clock()) / CLOCKS_PER_SEC - duration;
 
-		if(verbose)
-		{
-			fprintf(stderr, "Driver Events:%" PRIu64 "\nDriver Drops:%" PRIu64 "\n",
-				cstats.n_evts,
-				cstats.n_drops);
+			scap_stats cstats;
+			inspector->get_capture_stats(&cstats);
+
+			if(verbose)
+			{
+				fprintf(stderr, "Driver Events:%" PRIu64 "\nDriver Drops:%" PRIu64 "\n",
+					cstats.n_evts,
+					cstats.n_drops);
+
+				fprintf(stderr, "Elapsed time: %.3lf, Captured Events: %" PRIu64 ", %.2lf eps\n",
+					duration,
+					cinfo.m_nevts,
+					(double)cinfo.m_nevts / duration);
+			}
+
+			//
+			// Done. Close the capture.
+			//
+			inspector->close();
 
-			fprintf(stderr, "Elapsed time: %.3lf, Captured Events: %" PRIu64 ", %.2lf eps\n",
-				duration,
-				cinfo.m_nevts,
-				(double)cinfo.m_nevts / duration);
 		}
 	}
-	catch(sinsp_exception e)
+	catch(sinsp_exception& e)
 	{
 		cerr << e.what() << endl;
-		res = EXIT_FAILURE;
+		handle_end_of_file(print_progress);
+		res.m_res = EXIT_FAILURE;
 	}
 	catch(...)
 	{
-		res = EXIT_FAILURE;
+		handle_end_of_file(print_progress);
+		res.m_res = EXIT_FAILURE;
 	}
 
 exit:
+	//
+	// If any of the chisels is requesting another run,
+	//
+	for(vector<sinsp_chisel*>::iterator it = g_chisels.begin();
+		it != g_chisels.end(); ++it)
+	{
+		string na;
+		if((*it)->get_nextrun_args(&na))
+		{
+			res.m_next_run_args = sinsp_split(na, ' ');
+		}
+	}
 
 	//
 	// If there's a summary table, sort and print it
@@ -880,12 +1214,9 @@ exit:
 	}
 
 	//
-	// Free the chisels
+	// Free all the stuff that was allocated
 	//
-	for(vector<sinsp_chisel*>::iterator it = chisels.begin(); it != chisels.end(); ++it)
-	{
-		delete *it;
-	}
+	free_chisels();
 
 	if(inspector)
 	{
@@ -897,9 +1228,42 @@ exit:
 		delete display_filter;
 	}
 
+	return res;
+}
+
+//
+// MAIN
+//
+int main(int argc, char **argv)
+{
+	sysdig_init_res res;
+
+	res = sysdig_init(argc, argv);
+
+	//
+	// Check if a second run has been requested
+	//
+	if(res.m_next_run_args.size() != 0)
+	{
+		optind = 1;
+		opterr = 1;
+		optopt = '?';
+
+		int newargc = (int)res.m_next_run_args.size() + 1;
+		vector<char*> newargv;
+
+		newargv.push_back(argv[0]);
+
+		for(int32_t j = 1; j < newargc; j++)
+		{
+			newargv.push_back((char*)res.m_next_run_args[j - 1].c_str());
+		}
+
+		res = sysdig_init(newargc, &(newargv[0]));
+	}
 #ifdef _WIN32
 	_CrtDumpMemoryLeaks();
 #endif
 
-	return res;
+	return res.m_res;
 }
